@@ -15,6 +15,7 @@ import awscrt.exceptions
 from awscrt.http import HttpProxyOptions, HttpRequest
 from awscrt.io import ClientBootstrap, ClientTlsContext, SocketOptions
 from dataclasses import dataclass
+from awscrt.mqtt5 import Client as Mqtt5Client
 
 
 class QoS(IntEnum):
@@ -48,6 +49,13 @@ class QoS(IntEnum):
     does not support QoS 2 at time of writing (May 2020).
     """
 
+    def to_mqtt5(self):
+        from awscrt.mqtt5 import QoS as Mqtt5QoS
+        """Convert a Mqtt3 QoS to Mqtt5 QoS
+
+        """
+        return Mqtt5QoS(self.value)
+
 
 def _try_qos(qos_value):
     """Return None if the value cannot be converted to Qos (ex: 0x80 subscribe failure)"""
@@ -127,6 +135,39 @@ class Will:
         self.retain = retain
 
 
+@dataclass
+class OnConnectionSuccessData:
+    """Dataclass containing data related to a on_connection_success Callback
+
+    Args:
+        return_code (ConnectReturnCode): Connect return. code received from the server.
+        session_present (bool): True if the connection resumes an existing session.
+                                False if new session. Note that the server has forgotten all previous subscriptions
+                                if this is False.
+                                Subscriptions can be re-established via resubscribe_existing_topics() if the connection was a reconnection.
+    """
+    return_code: ConnectReturnCode = None
+    session_present: bool = False
+
+
+@dataclass
+class OnConnectionFailureData:
+    """Dataclass containing data related to a on_connection_failure Callback
+
+    Args:
+        error (ConnectReturnCode): Error code with reason for connection failure
+    """
+    error: awscrt.exceptions.AwsCrtError = None
+
+
+@dataclass
+class OnConnectionClosedData:
+    """Dataclass containing data related to a on_connection_closed Callback.
+    Currently unused.
+    """
+    pass
+
+
 class Client(NativeResource):
     """MQTT client.
 
@@ -187,7 +228,7 @@ class Connection(NativeResource):
             The `session_present` bool in the connection callback informs
             whether an existing session was successfully resumed.
             If an existing session is resumed, the server remembers previous subscriptions
-            and sends mesages (with QoS1 or higher) that were published while the client was offline.
+            and sends messages (with QoS1 or higher) that were published while the client was offline.
 
         on_connection_interrupted: Optional callback invoked whenever the MQTT connection is lost.
             The MQTT client will automatically attempt to reconnect.
@@ -213,6 +254,31 @@ class Connection(NativeResource):
 
                 *   `**kwargs` (dict): Forward-compatibility kwargs.
 
+        on_connection_success: Optional callback invoked whenever the connection successfully connects.
+            This callback is invoked for every successful connect and every successful reconnect.
+
+            Function should take the following arguments and return nothing:
+
+                * `connection` (:class:`Connection`): This MQTT Connection
+
+                * `callback_data` (:class:`OnConnectionSuccessData`): The data returned from the connection success.
+
+        on_connection_failure: Optional callback invoked whenever the connection fails to connect.
+            This callback is invoked for every failed connect and every failed reconnect.
+
+            Function should take the following arguments and return nothing:
+
+                * `connection` (:class:`Connection`): This MQTT Connection
+
+                * `callback_data` (:class:`OnConnectionFailureData`): The data returned from the connection failure.
+
+        on_connection_closed: Optional callback invoked whenever the connection has been disconnected and shutdown successfully.
+            Function should take the following arguments and return nothing:
+
+                * `connection` (:class:`Connection`): This MQTT Connection
+
+                * `callback_data` (:class:`OnConnectionClosedData`): The data returned from the connection close.
+
         reconnect_min_timeout_secs (int): Minimum time to wait between reconnect attempts.
             Must be <= `reconnect_max_timeout_secs`.
             Wait starts at min and doubles with each attempt until max is reached.
@@ -286,10 +352,13 @@ class Connection(NativeResource):
                  use_websockets=False,
                  websocket_proxy_options=None,
                  websocket_handshake_transform=None,
-                 proxy_options=None
+                 proxy_options=None,
+                 on_connection_success=None,
+                 on_connection_failure=None,
+                 on_connection_closed=None
                  ):
 
-        assert isinstance(client, Client)
+        assert isinstance(client, Client) or isinstance(client, Mqtt5Client)
         assert callable(on_connection_interrupted) or on_connection_interrupted is None
         assert callable(on_connection_resumed) or on_connection_resumed is None
         assert isinstance(will, Will) or will is None
@@ -297,6 +366,9 @@ class Connection(NativeResource):
         assert isinstance(websocket_proxy_options, HttpProxyOptions) or websocket_proxy_options is None
         assert isinstance(proxy_options, HttpProxyOptions) or proxy_options is None
         assert callable(websocket_handshake_transform) or websocket_handshake_transform is None
+        assert callable(on_connection_success) or on_connection_success is None
+        assert callable(on_connection_failure) or on_connection_failure is None
+        assert callable(on_connection_closed) or on_connection_closed is None
 
         if reconnect_min_timeout_secs > reconnect_max_timeout_secs:
             raise ValueError("'reconnect_min_timeout_secs' cannot exceed 'reconnect_max_timeout_secs'")
@@ -312,10 +384,14 @@ class Connection(NativeResource):
 
         # init-only
         self.client = client
+        self._client_version = 5 if isinstance(client, Mqtt5Client) else 3
         self._on_connection_interrupted_cb = on_connection_interrupted
         self._on_connection_resumed_cb = on_connection_resumed
         self._use_websockets = use_websockets
         self._ws_handshake_transform_cb = websocket_handshake_transform
+        self._on_connection_success_cb = on_connection_success
+        self._on_connection_failure_cb = on_connection_failure
+        self._on_connection_closed_cb = on_connection_closed
 
         # may be changed at runtime, take effect the the next time connect/reconnect occurs
         self.client_id = client_id
@@ -337,6 +413,7 @@ class Connection(NativeResource):
             self,
             client,
             use_websockets,
+            self._client_version
         )
 
     def _check_uses_old_message_callback_signature(self, callback):
@@ -385,6 +462,26 @@ class Connection(NativeResource):
             if not future.done():
                 transform_args.set_done(e)
 
+    def _on_connection_closed(self):
+        if self:
+            if self._on_connection_closed_cb:
+                data = OnConnectionClosedData()
+                self._on_connection_closed_cb(connection=self, callback_data=data)
+
+    def _on_connection_success(self, return_code, session_present):
+        if self:
+            if self._on_connection_success_cb:
+                data = OnConnectionSuccessData(
+                    return_code=ConnectReturnCode(return_code),
+                    session_present=session_present)
+                self._on_connection_success_cb(connection=self, callback_data=data)
+
+    def _on_connection_failure(self, error_code):
+        if self:
+            if self._on_connection_failure_cb:
+                data = OnConnectionFailureData(error=awscrt.exceptions.from_code(error_code))
+                self._on_connection_failure_cb(connection=self, callback_data=data)
+
     def connect(self):
         """Open the actual connection to the server (async).
 
@@ -559,6 +656,9 @@ class Connection(NativeResource):
 
         try:
             assert callable(callback) or callback is None
+            from awscrt.mqtt5 import QoS as Mqtt5QoS
+            if (isinstance(qos, Mqtt5QoS)):
+                qos = qos.to_mqtt3()
             assert isinstance(qos, QoS)
             packet_id = _awscrt.mqtt_client_connection_subscribe(
                 self._binding, topic, qos.value, callback_wrapper, suback)
@@ -723,6 +823,10 @@ class Connection(NativeResource):
                 future.set_result(dict(packet_id=packet_id))
 
         try:
+            from awscrt.mqtt5 import QoS as Mqtt5QoS
+            if (isinstance(qos, Mqtt5QoS)):
+                qos = qos.to_mqtt3()
+            assert isinstance(qos, QoS)
             packet_id = _awscrt.mqtt_client_connection_publish(self._binding, topic, payload, qos.value, retain, puback)
         except Exception as e:
             future.set_exception(e)
