@@ -12,7 +12,7 @@
 
 AWS_PUSH_SANE_WARNING_LEVEL
 
-#define AWS_ARRAY_LIST_DEBUG_FILL 0xDD
+enum { AWS_ARRAY_LIST_DEBUG_FILL = 0xDD };
 
 struct aws_array_list {
     struct aws_allocator *alloc;
@@ -59,6 +59,21 @@ void aws_array_list_init_static(
     size_t item_count,
     size_t item_size);
 
+/**
+ * Initializes an array list with a preallocated array of *already-initialized* elements. item_count is the number of
+ * elements in the array, and item_size is the size in bytes of each element.
+ *
+ * Once initialized, nothing further can be added to the list, since it will be full and cannot resize.
+ *
+ * Primary use case is to treat an already-initialized C array as an array list.
+ */
+AWS_STATIC_IMPL
+void aws_array_list_init_static_from_initialized(
+    struct aws_array_list *AWS_RESTRICT list,
+    void *raw_array,
+    size_t item_count,
+    size_t item_size);
+
 /**
  * Set of properties of a valid aws_array_list.
  */
