@@ -18,7 +18,8 @@
 #    define AWS_PUSH_SANE_WARNING_LEVEL                                                                                \
         __pragma(warning(push))                                                                                        \
         __pragma(warning(disable : 4820)) /* padding added to struct */                                                \
-        __pragma(warning(disable : 4514)) /* unreferenced inline function has been removed */
+        __pragma(warning(disable : 4514)) /* unreferenced inline function has been removed */                          \
+        __pragma(warning(disable : 5039)) /* reference to potentially throwing function passed to extern C function */
 #    define AWS_POP_SANE_WARNING_LEVEL __pragma(warning(pop))
 #else
 #    define AWS_PUSH_SANE_WARNING_LEVEL
@@ -60,7 +61,7 @@ AWS_STATIC_ASSERT(CALL_OVERLOAD_TEST(1) == 1);
 AWS_STATIC_ASSERT(CALL_OVERLOAD_TEST(1, 2) == 2);
 AWS_STATIC_ASSERT(CALL_OVERLOAD_TEST(1, 2, 3) == 3);
 
-#define AWS_CACHE_LINE 64
+enum { AWS_CACHE_LINE = 64 };
 /**
  * Format macro for strings of a specified length.
  * Allows non null-terminated strings to be used with the printf family of functions.
