@@ -8,6 +8,18 @@
 #include <aws/common/string.h>
 #include <aws/testing/aws_test_harness.h>
 
+static int s_array_list_zero_length(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+    (void)allocator;
+
+    struct aws_array_list list;
+    AWS_ZERO_STRUCT(list);
+    ASSERT_INT_EQUALS(0, aws_array_list_length(&list));
+    aws_array_list_clear(&list);
+    return AWS_OP_SUCCESS;
+}
+AWS_TEST_CASE(array_list_zero_length, s_array_list_zero_length)
+
 static int s_array_list_order_push_back_pop_front_fn(struct aws_allocator *allocator, void *ctx) {
 
     (void)ctx;
