@@ -29,7 +29,9 @@
 #endif
 
 AWS_STATIC_STRING_FROM_LITERAL(s_http_proxy_env_var, "HTTP_PROXY");
+AWS_STATIC_STRING_FROM_LITERAL(s_http_proxy_env_var_low, "http_proxy");
 AWS_STATIC_STRING_FROM_LITERAL(s_https_proxy_env_var, "HTTPS_PROXY");
+AWS_STATIC_STRING_FROM_LITERAL(s_https_proxy_env_var_low, "https_proxy");
 
 enum new_connection_result_type {
     AWS_NCRT_SUCCESS,
@@ -153,7 +155,7 @@ static int s_cm_tester_init(struct cm_tester_options *options) {
 
     aws_io_clock_fn *clock_fn = &aws_high_res_clock_get_ticks;
     if (options->mock_table) {
-        clock_fn = options->mock_table->get_monotonic_time;
+        clock_fn = options->mock_table->aws_high_res_clock_get_ticks;
     }
 
     tester->event_loop_group = aws_event_loop_group_new(tester->allocator, clock_fn, 1, s_new_event_loop, NULL, NULL);
@@ -302,7 +304,7 @@ release:
 
             if (close_first) {
                 if (tester->mock_table) {
-                    tester->mock_table->close_connection(connection);
+                    tester->mock_table->aws_http_connection_close(connection);
                 } else {
                     aws_http_connection_close(connection);
                 }
@@ -465,6 +467,7 @@ static int s_test_connection_manager_single_connection(struct aws_allocator *all
     ASSERT_SUCCESS(s_wait_on_connection_reply_count(1));
 
     ASSERT_SUCCESS(s_release_connections(1, false));
+    ASSERT_UINT_EQUALS(0, s_tester.connection_errors);
 
     ASSERT_SUCCESS(s_cm_tester_clean_up());
 
@@ -472,6 +475,39 @@ static int s_test_connection_manager_single_connection(struct aws_allocator *all
 }
 AWS_TEST_CASE(test_connection_manager_single_connection, s_test_connection_manager_single_connection);
 
+static int s_test_connection_manager_proxy_envrionment_empty_string(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+    /* Set proxy related envrionment variables to empty string and make sure we just skip proxy */
+    struct aws_string *empty = aws_string_new_from_c_str(allocator, "");
+    ASSERT_SUCCESS(aws_set_environment_value(s_http_proxy_env_var, empty));
+    ASSERT_SUCCESS(aws_set_environment_value(s_http_proxy_env_var_low, empty));
+    ASSERT_SUCCESS(aws_set_environment_value(s_https_proxy_env_var, empty));
+    ASSERT_SUCCESS(aws_set_environment_value(s_https_proxy_env_var_low, empty));
+
+    struct cm_tester_options options = {
+        .allocator = allocator,
+        .max_connections = 5,
+        .use_proxy_env = true,
+    };
+
+    ASSERT_SUCCESS(s_cm_tester_init(&options));
+
+    s_acquire_connections(1);
+
+    ASSERT_SUCCESS(s_wait_on_connection_reply_count(1));
+
+    ASSERT_SUCCESS(s_release_connections(1, false));
+    ASSERT_UINT_EQUALS(0, s_tester.connection_errors);
+
+    ASSERT_SUCCESS(s_cm_tester_clean_up());
+    aws_string_destroy(empty);
+
+    return AWS_OP_SUCCESS;
+}
+AWS_TEST_CASE(
+    test_connection_manager_proxy_envrionment_empty_string,
+    s_test_connection_manager_proxy_envrionment_empty_string);
+
 static int s_test_connection_manager_single_http2_connection(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
 
@@ -513,6 +549,7 @@ static int s_test_connection_manager_single_http2_connection_failed(struct aws_a
     ASSERT_SUCCESS(s_wait_on_connection_reply_count(1));
 
     ASSERT_SUCCESS(s_release_connections(1, false));
+    ASSERT_UINT_EQUALS(1, s_tester.connection_errors);
 
     ASSERT_SUCCESS(s_cm_tester_clean_up());
 
@@ -543,6 +580,7 @@ static int s_test_connection_manager_single_http2_connection_with_settings(struc
     ASSERT_SUCCESS(s_wait_on_connection_reply_count(1));
 
     ASSERT_SUCCESS(s_release_connections(1, false));
+    ASSERT_UINT_EQUALS(0, s_tester.connection_errors);
 
     ASSERT_SUCCESS(s_cm_tester_clean_up());
 
@@ -567,6 +605,7 @@ static int s_test_connection_manager_many_connections(struct aws_allocator *allo
     ASSERT_SUCCESS(s_wait_on_connection_reply_count(20));
 
     ASSERT_SUCCESS(s_release_connections(20, false));
+    ASSERT_UINT_EQUALS(0, s_tester.connection_errors);
 
     ASSERT_SUCCESS(s_cm_tester_clean_up());
 
@@ -591,6 +630,7 @@ static int s_test_connection_manager_many_http2_connections(struct aws_allocator
     ASSERT_SUCCESS(s_wait_on_connection_reply_count(20));
 
     ASSERT_SUCCESS(s_release_connections(20, false));
+    ASSERT_UINT_EQUALS(0, s_tester.connection_errors);
 
     ASSERT_SUCCESS(s_cm_tester_clean_up());
 
@@ -617,6 +657,7 @@ static int s_test_connection_manager_acquire_release(struct aws_allocator *alloc
 
         ASSERT_SUCCESS(s_wait_on_connection_reply_count(i + 1));
     }
+    ASSERT_UINT_EQUALS(0, s_tester.connection_errors);
 
     ASSERT_SUCCESS(s_cm_tester_clean_up());
 
@@ -643,6 +684,7 @@ static int s_test_connection_manager_close_and_release(struct aws_allocator *all
 
         ASSERT_SUCCESS(s_wait_on_connection_reply_count(i + 1));
     }
+    ASSERT_UINT_EQUALS(0, s_tester.connection_errors);
 
     ASSERT_SUCCESS(s_cm_tester_clean_up());
 
@@ -675,6 +717,7 @@ static int s_test_connection_manager_acquire_release_mix(struct aws_allocator *a
 
         ASSERT_SUCCESS(s_wait_on_connection_reply_count(i + 1));
     }
+    ASSERT_UINT_EQUALS(0, s_tester.connection_errors);
 
     ASSERT_SUCCESS(s_cm_tester_clean_up());
 
@@ -766,14 +809,14 @@ static enum aws_http_version s_aws_http_connection_manager_connection_get_versio
 }
 
 static struct aws_http_connection_manager_system_vtable s_synchronous_mocks = {
-    .create_connection = s_aws_http_connection_manager_create_connection_sync_mock,
-    .release_connection = s_aws_http_connection_manager_release_connection_sync_mock,
-    .close_connection = s_aws_http_connection_manager_close_connection_sync_mock,
-    .is_connection_available = s_aws_http_connection_manager_is_connection_available_sync_mock,
-    .get_monotonic_time = aws_high_res_clock_get_ticks,
-    .connection_get_channel = s_aws_http_connection_manager_connection_get_channel_sync_mock,
-    .is_callers_thread = s_aws_http_connection_manager_is_callers_thread_sync_mock,
-    .connection_get_version = s_aws_http_connection_manager_connection_get_version_sync_mock,
+    .aws_http_client_connect = s_aws_http_connection_manager_create_connection_sync_mock,
+    .aws_http_connection_release = s_aws_http_connection_manager_release_connection_sync_mock,
+    .aws_http_connection_close = s_aws_http_connection_manager_close_connection_sync_mock,
+    .aws_http_connection_new_requests_allowed = s_aws_http_connection_manager_is_connection_available_sync_mock,
+    .aws_high_res_clock_get_ticks = aws_high_res_clock_get_ticks,
+    .aws_http_connection_get_channel = s_aws_http_connection_manager_connection_get_channel_sync_mock,
+    .aws_channel_thread_is_callers_thread = s_aws_http_connection_manager_is_callers_thread_sync_mock,
+    .aws_http_connection_get_version = s_aws_http_connection_manager_connection_get_version_sync_mock,
 };
 
 static int s_test_connection_manager_acquire_release_mix_synchronous(struct aws_allocator *allocator, void *ctx) {
@@ -891,14 +934,14 @@ static int s_test_connection_manager_proxy_setup_shutdown(struct aws_allocator *
 AWS_TEST_CASE(test_connection_manager_proxy_setup_shutdown, s_test_connection_manager_proxy_setup_shutdown);
 
 static struct aws_http_connection_manager_system_vtable s_idle_mocks = {
-    .create_connection = s_aws_http_connection_manager_create_connection_sync_mock,
-    .release_connection = s_aws_http_connection_manager_release_connection_sync_mock,
-    .close_connection = s_aws_http_connection_manager_close_connection_sync_mock,
-    .is_connection_available = s_aws_http_connection_manager_is_connection_available_sync_mock,
-    .get_monotonic_time = s_tester_get_mock_time,
-    .connection_get_channel = s_aws_http_connection_manager_connection_get_channel_sync_mock,
-    .is_callers_thread = s_aws_http_connection_manager_is_callers_thread_sync_mock,
-    .connection_get_version = s_aws_http_connection_manager_connection_get_version_sync_mock,
+    .aws_http_client_connect = s_aws_http_connection_manager_create_connection_sync_mock,
+    .aws_http_connection_release = s_aws_http_connection_manager_release_connection_sync_mock,
+    .aws_http_connection_close = s_aws_http_connection_manager_close_connection_sync_mock,
+    .aws_http_connection_new_requests_allowed = s_aws_http_connection_manager_is_connection_available_sync_mock,
+    .aws_high_res_clock_get_ticks = s_tester_get_mock_time,
+    .aws_http_connection_get_channel = s_aws_http_connection_manager_connection_get_channel_sync_mock,
+    .aws_channel_thread_is_callers_thread = s_aws_http_connection_manager_is_callers_thread_sync_mock,
+    .aws_http_connection_get_version = s_aws_http_connection_manager_connection_get_version_sync_mock,
 };
 
 static int s_register_acquired_connections(struct aws_array_list *seen_connections) {
@@ -1378,7 +1421,7 @@ static struct aws_byte_cursor s_get_proxy_host_for_test(
     bool h2) {
     struct aws_string *host_string;
     if (h2) {
-        host_string = configs->https_proxy_host_h2;
+        host_string = configs->https_proxy_host;
     } else if (auth_type == AWS_HPAT_BASIC) {
         host_string = configs->http_proxy_basic_host;
     } else if (proxy_test_type == TUNNELING_DOUBLE_TLS) {
@@ -1389,7 +1432,7 @@ static struct aws_byte_cursor s_get_proxy_host_for_test(
     return aws_byte_cursor_from_string(host_string);
 }
 
-static uint16_t s_get_proxy_port_for_test(
+static uint32_t s_get_proxy_port_for_test(
     struct proxy_integration_configurations *configs,
     enum proxy_test_type proxy_test_type,
     enum aws_http_proxy_authentication_type auth_type,
@@ -1404,7 +1447,7 @@ static uint16_t s_get_proxy_port_for_test(
     } else {
         port_string = configs->http_proxy_port;
     }
-    return (uint16_t)atoi(aws_string_c_str(port_string));
+    return (uint32_t)atoi(aws_string_c_str(port_string));
 }
 static struct aws_string *s_get_proxy_url_for_test(
     struct proxy_integration_configurations *configs,
