@@ -12,12 +12,15 @@
 #include <aws/s3/private/s3_meta_request_impl.h>
 #include <aws/s3/s3.h>
 #include <aws/s3/s3_client.h>
+#include <aws/s3/s3express_credentials_provider.h>
 
 #include <aws/common/common.h>
 #include <aws/common/condition_variable.h>
+#include <aws/common/file.h>
 #include <aws/common/logging.h>
 #include <aws/common/mutex.h>
 #include <aws/common/string.h>
+#include <aws/testing/async_stream_tester.h>
 
 struct aws_client_bootstrap;
 struct aws_credentials_provider;
@@ -85,6 +88,8 @@ struct aws_s3_tester {
     struct aws_client_bootstrap *client_bootstrap;
     struct aws_credentials_provider *credentials_provider;
     struct aws_signing_config_aws default_signing_config;
+    struct aws_credentials *anonymous_creds;
+    struct aws_signing_config_aws anonymous_signing_config;
 
     struct aws_condition_variable signal;
     bool bound_to_client;
@@ -95,6 +100,8 @@ struct aws_s3_tester {
 
     struct aws_string *bucket_name;
     struct aws_string *public_bucket_name;
+    struct aws_string *s3express_bucket_usw2_az1_endpoint;
+    struct aws_string *s3express_bucket_use1_az4_endpoint;
 
     struct {
         struct aws_mutex lock;
@@ -157,20 +164,27 @@ struct aws_s3_tester_meta_request_options {
 
     /* override client signing config */
     struct aws_signing_config_aws *signing_config;
+    /* use S3 Express signing config */
+    bool use_s3express_signing;
 
     aws_s3_meta_request_headers_callback_fn *headers_callback;
     aws_s3_meta_request_receive_body_callback_fn *body_callback;
     aws_s3_meta_request_finish_fn *finish_callback;
+    aws_s3_meta_request_progress_fn *progress_callback;
+    aws_s3_meta_request_upload_review_fn *upload_review_callback;
 
     /* Default Meta Request specific options. */
     struct {
         enum aws_s3_tester_default_type_mode mode;
+        struct aws_byte_cursor operation_name;
     } default_type_options;
 
     /* Get Object Meta Request specific options.*/
     struct {
         struct aws_byte_cursor object_path;
         struct aws_byte_cursor object_range;
+        /* Get the part from S3, starts from 1. 0 means not set. */
+        int part_number;
     } get_options;
 
     /* Put Object Meta request specific options. */
@@ -178,15 +192,22 @@ struct aws_s3_tester_meta_request_options {
         struct aws_byte_cursor object_path_override;
         uint32_t object_size_mb;
         bool ensure_multipart;
+        bool async_input_stream; /* send via async stream */
+        enum aws_async_read_completion_strategy async_read_strategy;
+        size_t max_bytes_per_read; /* test an input-stream read() that doesn't always fill the buffer */
+        bool file_on_disk;         /* write to file on disk, then send via aws_s3_meta_request_options.send_filepath */
+        /* If false, EOF is reported by the read() which produces the last few bytes.
+         * If true, EOF isn't reported until there's one more read(), producing zero bytes.
+         * This emulates an underlying stream that reports EOF by reading 0 bytes */
+        bool eof_requires_extra_read;
         bool invalid_request;
         bool invalid_input_stream;
         bool valid_md5;
         bool invalid_md5;
-        /* write file to desk, and then send via aws_s3_meta_request_options.send_filepath */
-        bool file_on_disk;
         struct aws_s3_meta_request_resume_token *resume_token;
         /* manually overwrite the content length for some invalid input stream */
         size_t content_length;
+        bool skip_content_length;
         struct aws_byte_cursor content_encoding;
     } put_options;
 
@@ -196,20 +217,24 @@ struct aws_s3_tester_meta_request_options {
     uint32_t dont_wait_for_shutdown : 1;
 
     uint32_t mrap_test : 1;
+
+    uint64_t *object_size_hint;
 };
 
 /* TODO Rename to something more generic such as "aws_s3_meta_request_test_data" */
 struct aws_s3_meta_request_test_results {
+    struct aws_allocator *allocator;
     struct aws_s3_tester *tester;
 
     aws_s3_meta_request_headers_callback_fn *headers_callback;
     aws_s3_meta_request_receive_body_callback_fn *body_callback;
     aws_s3_meta_request_finish_fn *finish_callback;
-    aws_s3_meta_request_shutdown_fn *shutdown_callback;
     aws_s3_meta_request_progress_fn *progress_callback;
+    aws_s3_meta_request_upload_review_fn *upload_review_callback;
 
     struct aws_http_headers *error_response_headers;
     struct aws_byte_buf error_response_body;
+    struct aws_string *error_response_operation_name;
     size_t part_size;
 
     int headers_response_status;
@@ -222,14 +247,26 @@ struct aws_s3_meta_request_test_results {
     int finished_error_code;
     enum aws_s3_checksum_algorithm algorithm;
 
-    /* accumulator of amount of bytes uploaded */
-    struct aws_atomic_var total_bytes_uploaded;
+    /* Record data from progress_callback() */
+    struct {
+        uint64_t content_length;          /* Remember progress->content_length */
+        uint64_t total_bytes_transferred; /* Accumulator for progress->bytes_transferred */
+    } progress;
 
     /* Protected the tester->synced_data.lock */
     struct {
         /* The array_list of `struct aws_s3_request_metrics *` */
         struct aws_array_list metrics;
     } synced_data;
+
+    /* record data from the upload_review_callback */
+    struct {
+        size_t invoked_count;
+        enum aws_s3_checksum_algorithm checksum_algorithm;
+        size_t part_count;
+        uint64_t *part_sizes_array;
+        struct aws_string **part_checksums_array;
+    } upload_review;
 };
 
 struct aws_s3_client_config;
@@ -311,8 +348,8 @@ struct aws_http_message *aws_s3_test_get_object_request_new(
 struct aws_http_message *aws_s3_test_put_object_request_new(
     struct aws_allocator *allocator,
     struct aws_byte_cursor *host,
-    struct aws_byte_cursor content_type,
     struct aws_byte_cursor key,
+    struct aws_byte_cursor content_type,
     struct aws_input_stream *body_stream,
     uint32_t flags);
 
@@ -410,8 +447,6 @@ enum aws_s3_test_stream_value {
     TEST_STREAM_VALUE_2,
 };
 
-struct aws_input_stream *aws_s3_bad_input_stream_new(struct aws_allocator *allocator, size_t length);
-
 struct aws_input_stream *aws_s3_test_input_stream_new(struct aws_allocator *allocator, size_t length);
 
 struct aws_input_stream *aws_s3_test_input_stream_new_with_value_type(
@@ -425,8 +460,20 @@ int aws_s3_tester_upload_file_path_init(
     struct aws_byte_buf *out_path_buffer,
     struct aws_byte_cursor file_path);
 
+/* Create a file on disk based on the input stream. Return the file path */
+struct aws_string *aws_s3_tester_create_file(
+    struct aws_allocator *allocator,
+    struct aws_byte_cursor test_object_path,
+    struct aws_input_stream *input_stream);
+
 int aws_s3_tester_get_content_length(const struct aws_http_headers *headers, uint64_t *out_content_length);
 
+int aws_s3_tester_check_s3express_creds_for_default_mock_response(struct aws_credentials *credentials);
+
+struct aws_parallel_input_stream *aws_parallel_input_stream_new_from_file_failure_tester(
+    struct aws_allocator *allocator,
+    struct aws_byte_cursor file_name);
+
 extern struct aws_s3_client_vtable g_aws_s3_client_mock_vtable;
 
 extern const struct aws_byte_cursor g_mock_server_uri;
@@ -448,5 +495,13 @@ extern struct aws_byte_cursor g_test_bucket_name;
  * aws-c-s3-test-bucket-public
  */
 extern struct aws_byte_cursor g_test_public_bucket_name;
+/* If `$CRT_S3_TEST_BUCKET_NAME` environment variable is set, use
+ * `$CRT_S3_TEST_BUCKET_NAME--usw2-az1--x-s3.s3express-usw2-az1.us-west-2.amazonaws.com`; otherwise, use
+ * aws-c-s3-test-bucket--usw2-az1--x-s3.s3express-usw2-az1.us-west-2.amazonaws.com */
+extern struct aws_byte_cursor g_test_s3express_bucket_usw2_az1_endpoint;
+/* If `$CRT_S3_TEST_BUCKET_NAME` environment variable is set, use
+ * `$CRT_S3_TEST_BUCKET_NAME--us1-az1--x-s3.s3express-use1-az4.us-east-1.amazonaws.com`; otherwise, use
+ * aws-c-s3-test-bucket--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com */
+extern struct aws_byte_cursor g_test_s3express_bucket_use1_az4_endpoint;
 
 #endif /* AWS_S3_TESTER_H */
