@@ -274,6 +274,7 @@ static int eval_expected(struct aws_allocator *allocator, struct aws_byte_cursor
                 aws_json_value_get_from_object(endpoint, aws_byte_cursor_from_c_str("url"));
             struct aws_byte_cursor expected_url;
             aws_json_value_get_string(expected_url_node, &expected_url);
+            AWS_LOGF_DEBUG(0, PRInSTR " " PRInSTR, AWS_BYTE_CURSOR_PRI(url), AWS_BYTE_CURSOR_PRI(expected_url));
             ASSERT_TRUE(aws_byte_cursor_eq(&url, &expected_url));
 
             struct aws_byte_cursor properties;
@@ -475,3 +476,12 @@ static int s_test_condition_mem_clean_up(struct aws_allocator *allocator, void *
 
     return AWS_OP_SUCCESS;
 }
+
+AWS_TEST_CASE(test_endpoints_custom, s_test_endpoints_custom)
+static int s_test_endpoints_custom(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+
+    ASSERT_SUCCESS(eval_expected(allocator, aws_byte_cursor_from_c_str("custom_partition.json")));
+
+    return AWS_OP_SUCCESS;
+}
