@@ -4,7 +4,7 @@
 use criterion::{criterion_group, criterion_main, Criterion};
 use s2n_tls::{
     security,
-    testing::{build_config, s2n_tls_pair},
+    testing::{build_config, establish_connection},
 };
 
 pub fn handshake(c: &mut Criterion) {
@@ -13,9 +13,9 @@ pub fn handshake(c: &mut Criterion) {
     for policy in security::ALL_POLICIES {
         let config = build_config(policy).unwrap();
         group.bench_function(format!("handshake_{:?}", policy), move |b| {
-            // This does include connection initalization overhead.
-            // TODO: create a separate benchamrk that excludes this step.
-            b.iter(|| s2n_tls_pair(config.clone()));
+            // This does include connection initialization overhead.
+            // TODO: create a separate benchmark that excludes this step.
+            b.iter(|| establish_connection(config.clone()));
         });
     }
 
