@@ -12,7 +12,10 @@ use s2n_tls_sys::*;
 use std::{
     ffi::{c_void, CString},
     path::Path,
+    pin::Pin,
     sync::atomic::{AtomicUsize, Ordering},
+    task::Poll,
+    time::{Duration, SystemTime},
 };
 
 #[derive(Debug, PartialEq)]
@@ -149,12 +152,16 @@ impl Drop for Config {
 }
 
 #[derive(Default)]
-pub struct Builder(Config);
+pub struct Builder {
+    config: Config,
+    load_system_certs: bool,
+    enable_ocsp: bool,
+}
 
 impl Builder {
     pub fn new() -> Self {
         crate::init::init();
-        let config = unsafe { s2n_config_new().into_result() }.unwrap();
+        let config = unsafe { s2n_config_new_minimal().into_result() }.unwrap();
 
         let context = Box::<Context>::default();
         let context = Box::into_raw(context) as *mut c_void;
@@ -175,7 +182,11 @@ impl Builder {
             .unwrap();
         }
 
-        Self(Config(config))
+        Self {
+            config: Config(config),
+            load_system_certs: true,
+            enable_ocsp: false,
+        }
     }
 
     pub fn set_alert_behavior(&mut self, value: AlertBehavior) -> Result<&mut Self, Error> {
@@ -282,6 +293,10 @@ impl Builder {
         Ok(self)
     }
 
+    /// Adds to the trust store from a CA file or directory containing trusted certificates.
+    ///
+    /// NOTE: This function is equivalent to `s2n_config_set_verification_ca_location` except it does
+    /// not automatically enable the client to request OCSP stapling from the server.
     pub fn trust_location(
         &mut self,
         file: Option<&Path>,
@@ -314,6 +329,25 @@ impl Builder {
             s2n_config_set_verification_ca_location(self.as_mut_ptr(), file_ptr, dir_ptr)
                 .into_result()
         }?;
+
+        // If OCSP has not been explicitly requested, turn off OCSP. This is to prevent this function from
+        // automatically enabling `OCSP` due to the legacy behavior of `s2n_config_set_verification_ca_location`
+        if !self.enable_ocsp {
+            unsafe {
+                s2n_config_set_status_request_type(self.as_mut_ptr(), s2n_status_request_type::NONE)
+                    .into_result()?
+            };
+        }
+
+        Ok(self)
+    }
+
+    /// Sets whether or not default system certificates will be loaded into the trust store.
+    ///
+    /// Set to false for increased performance if system certificates are not needed during
+    /// certificate validation.
+    pub fn with_system_certs(&mut self, load_system_certs: bool) -> Result<&mut Self, Error> {
+        self.load_system_certs = load_system_certs;
         Ok(self)
     }
 
@@ -338,6 +372,7 @@ impl Builder {
             s2n_config_set_status_request_type(self.as_mut_ptr(), s2n_status_request_type::OCSP)
                 .into_result()
         }?;
+        self.enable_ocsp = true;
         Ok(self)
     }
 
@@ -387,12 +422,12 @@ impl Builder {
             verify_host(host_name, host_name_len, handler)
         }
 
-        self.0.context_mut().verify_host_callback = Some(Box::new(handler));
+        self.config.context_mut().verify_host_callback = Some(Box::new(handler));
         unsafe {
             s2n_config_set_verify_host_callback(
                 self.as_mut_ptr(),
                 Some(verify_host_cb_fn),
-                self.0.context_mut() as *mut Context as *mut c_void,
+                self.config.context_mut() as *mut Context as *mut c_void,
             )
             .into_result()?;
         }
@@ -441,7 +476,7 @@ impl Builder {
         }
 
         let handler = Box::new(handler);
-        let context = self.0.context_mut();
+        let context = self.config.context_mut();
         context.client_hello_callback = Some(handler);
 
         unsafe {
@@ -456,6 +491,55 @@ impl Builder {
         Ok(self)
     }
 
+    pub fn set_connection_initializer<T: 'static + ConnectionInitializer>(
+        &mut self,
+        handler: T,
+    ) -> Result<&mut Self, Error> {
+        // Store callback in config context
+        let handler = Box::new(handler);
+        let context = self.config.context_mut();
+        context.connection_initializer = Some(handler);
+        Ok(self)
+    }
+
+    /// Sets a custom callback which provides access to session tickets when they arrive
+    pub fn set_session_ticket_callback<T: 'static + SessionTicketCallback>(
+        &mut self,
+        handler: T,
+    ) -> Result<&mut Self, Error> {
+        // enable session tickets automatically
+        self.enable_session_tickets(true)?;
+
+        // Define C callback function that can be set on the s2n_config struct
+        unsafe extern "C" fn session_ticket_cb(
+            conn_ptr: *mut s2n_connection,
+            _context: *mut ::libc::c_void,
+            session_ticket: *mut s2n_session_ticket,
+        ) -> libc::c_int {
+            let session_ticket = SessionTicket::from_ptr(&*session_ticket);
+            with_context(conn_ptr, |conn, context| {
+                let callback = context.session_ticket_callback.as_ref();
+                callback.map(|c| c.on_session_ticket(conn, session_ticket))
+            });
+            CallbackResult::Success.into()
+        }
+
+        // Store callback in context
+        let handler = Box::new(handler);
+        let context = self.config.context_mut();
+        context.session_ticket_callback = Some(handler);
+
+        unsafe {
+            s2n_config_set_session_ticket_cb(
+                self.as_mut_ptr(),
+                Some(session_ticket_cb),
+                self.config.context_mut() as *mut Context as *mut c_void,
+            )
+            .into_result()
+        }?;
+        Ok(self)
+    }
+
     /// Set a callback function triggered by operations requiring the private key.
     ///
     /// See https://github.com/aws/s2n-tls/blob/main/docs/USAGE-GUIDE.md#private-key-operation-related-calls
@@ -479,7 +563,7 @@ impl Builder {
         }
 
         let handler = Box::new(handler);
-        let context = self.0.context_mut();
+        let context = self.config.context_mut();
         context.private_key_callback = Some(handler);
 
         unsafe {
@@ -513,13 +597,13 @@ impl Builder {
         }
 
         let handler = Box::new(handler);
-        let context = self.0.context_mut();
+        let context = self.config.context_mut();
         context.wall_clock = Some(handler);
         unsafe {
             s2n_config_set_wall_clock(
                 self.as_mut_ptr(),
                 Some(clock_cb),
-                self.0.context_mut() as *mut _ as *mut c_void,
+                self.config.context_mut() as *mut _ as *mut c_void,
             )
             .into_result()?;
         }
@@ -550,25 +634,104 @@ impl Builder {
         }
 
         let handler = Box::new(handler);
-        let context = self.0.context_mut();
+        let context = self.config.context_mut();
         context.monotonic_clock = Some(handler);
         unsafe {
             s2n_config_set_monotonic_clock(
                 self.as_mut_ptr(),
                 Some(clock_cb),
-                self.0.context_mut() as *mut _ as *mut c_void,
+                self.config.context_mut() as *mut _ as *mut c_void,
             )
             .into_result()?;
         }
         Ok(self)
     }
 
-    pub fn build(self) -> Result<Config, Error> {
-        Ok(self.0)
+    /// Enable negotiating session tickets in a TLS connection
+    pub fn enable_session_tickets(&mut self, enable: bool) -> Result<&mut Self, Error> {
+        unsafe {
+            s2n_config_set_session_tickets_onoff(self.as_mut_ptr(), enable.into()).into_result()
+        }?;
+        Ok(self)
+    }
+
+    /// Adds a key which will be used to encrypt and decrypt session tickets. The intro_time parameter is time since
+    /// the Unix epoch (Midnight, January 1st, 1970). The key must be at least 16 bytes.
+    pub fn add_session_ticket_key(
+        &mut self,
+        key_name: &[u8],
+        key: &[u8],
+        intro_time: SystemTime,
+    ) -> Result<&mut Self, Error> {
+        let key_name_len: u32 = key_name
+            .len()
+            .try_into()
+            .map_err(|_| Error::INVALID_INPUT)?;
+        let key_len: u32 = key.len().try_into().map_err(|_| Error::INVALID_INPUT)?;
+        let intro_time = intro_time
+            .duration_since(std::time::UNIX_EPOCH)
+            .map_err(|_| Error::INVALID_INPUT)?;
+        // Ticket keys should be at least 128 bits in strength
+        // https://www.rfc-editor.org/rfc/rfc5077#section-5.5
+        if key_len < 16 {
+            return Err(Error::INVALID_INPUT);
+        }
+        self.enable_session_tickets(true)?;
+        unsafe {
+            s2n_config_add_ticket_crypto_key(
+                self.as_mut_ptr(),
+                key_name.as_ptr(),
+                key_name_len,
+                // s2n-tls doesn't mutate key, it's just mut for easier use with stuffers and blobs
+                key.as_ptr() as *mut u8,
+                key_len,
+                intro_time.as_secs(),
+            )
+            .into_result()
+        }?;
+        Ok(self)
+    }
+
+    // Sets how long a session ticket key will be able to be used for both encryption
+    // and decryption of tickets
+    pub fn set_ticket_key_encrypt_decrypt_lifetime(
+        &mut self,
+        lifetime: Duration,
+    ) -> Result<&mut Self, Error> {
+        unsafe {
+            s2n_config_set_ticket_encrypt_decrypt_key_lifetime(
+                self.as_mut_ptr(),
+                lifetime.as_secs(),
+            )
+            .into_result()
+        }?;
+        Ok(self)
+    }
+
+    // Sets how long a session ticket key will be able to be used for only decryption
+    pub fn set_ticket_key_decrypt_lifetime(
+        &mut self,
+        lifetime: Duration,
+    ) -> Result<&mut Self, Error> {
+        unsafe {
+            s2n_config_set_ticket_decrypt_key_lifetime(self.as_mut_ptr(), lifetime.as_secs())
+                .into_result()
+        }?;
+        Ok(self)
+    }
+
+    pub fn build(mut self) -> Result<Config, Error> {
+        if self.load_system_certs {
+            unsafe {
+                s2n_config_load_system_certs(self.as_mut_ptr()).into_result()?;
+            }
+        }
+
+        Ok(self.config)
     }
 
     fn as_mut_ptr(&mut self) -> *mut s2n_config {
-        self.0.as_mut_ptr()
+        self.config.as_mut_ptr()
     }
 }
 
@@ -585,6 +748,8 @@ pub(crate) struct Context {
     pub(crate) client_hello_callback: Option<Box<dyn ClientHelloCallback>>,
     pub(crate) private_key_callback: Option<Box<dyn PrivateKeyCallback>>,
     pub(crate) verify_host_callback: Option<Box<dyn VerifyHostNameCallback>>,
+    pub(crate) session_ticket_callback: Option<Box<dyn SessionTicketCallback>>,
+    pub(crate) connection_initializer: Option<Box<dyn ConnectionInitializer>>,
     pub(crate) wall_clock: Option<Box<dyn WallClock>>,
     pub(crate) monotonic_clock: Option<Box<dyn MonotonicClock>>,
 }
@@ -600,8 +765,94 @@ impl Default for Context {
             client_hello_callback: None,
             private_key_callback: None,
             verify_host_callback: None,
+            session_ticket_callback: None,
+            connection_initializer: None,
             wall_clock: None,
             monotonic_clock: None,
         }
     }
 }
+
+/// A trait executed asynchronously before a new connection negotiates TLS.
+///
+/// Used for dynamic configuration of a specific connection.
+///
+/// # Safety: This trait is polled to completion at the beginning of the
+/// [connection::poll_negotiate](`crate::connection::poll_negotiate()`) function.
+/// Therefore, negotiation of the TLS connection will not begin until the Future has completed.
+pub trait ConnectionInitializer: 'static + Send + Sync {
+    /// The application can return an `Ok(None)` to resolve the callback
+    /// synchronously or return an `Ok(Some(ConnectionFuture))` if it wants to
+    /// run some asynchronous task before resolving the callback.
+    fn initialize_connection(
+        &self,
+        connection: &mut crate::connection::Connection,
+    ) -> ConnectionFutureResult;
+}
+
+impl<A: ConnectionInitializer, B: ConnectionInitializer> ConnectionInitializer for (A, B) {
+    fn initialize_connection(
+        &self,
+        connection: &mut crate::connection::Connection,
+    ) -> ConnectionFutureResult {
+        let a = self.0.initialize_connection(connection)?;
+        let b = self.1.initialize_connection(connection)?;
+        match (a, b) {
+            (None, None) => Ok(None),
+            (None, Some(fut)) => Ok(Some(fut)),
+            (Some(fut), None) => Ok(Some(fut)),
+            (Some(fut_a), Some(fut_b)) => Ok(Some(Box::pin(ConcurrentConnectionFuture::new([
+                fut_a, fut_b,
+            ])))),
+        }
+    }
+}
+
+struct ConcurrentConnectionFuture<const N: usize> {
+    futures: [Option<Pin<Box<dyn ConnectionFuture>>>; N],
+}
+
+impl<const N: usize> ConcurrentConnectionFuture<N> {
+    fn new(futures: [Pin<Box<dyn ConnectionFuture>>; N]) -> Self {
+        let futures = futures.map(Some);
+        Self { futures }
+    }
+}
+
+impl<const N: usize> ConnectionFuture for ConcurrentConnectionFuture<N> {
+    fn poll(
+        mut self: std::pin::Pin<&mut Self>,
+        connection: &mut crate::connection::Connection,
+        ctx: &mut core::task::Context,
+    ) -> std::task::Poll<Result<(), Error>> {
+        let mut is_pending = false;
+        for container in self.futures.iter_mut() {
+            if let Some(future) = container.as_mut() {
+                match future.as_mut().poll(connection, ctx) {
+                    Poll::Ready(result) => {
+                        result?;
+                        *container = None;
+                    }
+                    Poll::Pending => is_pending = true,
+                }
+            }
+        }
+        if is_pending {
+            Poll::Pending
+        } else {
+            Poll::Ready(Ok(()))
+        }
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    // ensure the config context is send and sync
+    #[test]
+    fn context_send_sync_test() {
+        fn assert_send_sync<T: 'static + Send + Sync>() {}
+        assert_send_sync::<Context>();
+    }
+}
