@@ -41,7 +41,7 @@ mv s2n aws-crt-cpp/crt/
 
 cmake ./aws-crt-cpp -Bbuild -GNinja -DBUILD_DEPS=ON -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX="${INSTALL_DIR}"
 ninja -C ./build install
-CTEST_PARALLEL_LEVEL=$(nproc) ninja -C ./build test
+CTEST_OUTPUT_ON_FAILURE=1 CTEST_PARALLEL_LEVEL=$(nproc) ninja -C ./build test
 
 popd
 
