@@ -16,6 +16,7 @@ PROTOCOLS = [
     Protocols.TLS12,
     Protocols.TLS11,
     Protocols.TLS10,
+    Protocols.SSLv3,
 ]
 
 
@@ -52,7 +53,8 @@ ALL_TEST_CERTS = [
     Certificates.RSA_4096_SHA512,
     Certificates.ECDSA_256,
     Certificates.ECDSA_384,
-    Certificates.RSA_PSS_2048_SHA256
+    Certificates.ECDSA_521,
+    Certificates.RSA_PSS_2048_SHA256,
 ]
 
 
