@@ -320,3 +320,72 @@ def test_s2nd_falls_back_to_full_connection(managed_process, tmp_path, cipher, c
         assert b'Resumed session' not in results.stdout
         assert to_bytes("Actual protocol version: {}".format(
             s2n_version)) in results.stdout
+
+
+@pytest.mark.uncollect_if(func=invalid_test_parameters)
+@pytest.mark.parametrize("cipher", ALL_TEST_CIPHERS, ids=get_parameter_name)
+@pytest.mark.parametrize("curve", ALL_TEST_CURVES, ids=get_parameter_name)
+@pytest.mark.parametrize("certificate", ALL_TEST_CERTS, ids=get_parameter_name)
+@pytest.mark.parametrize("protocol", [p for p in PROTOCOLS if p < Protocols.TLS13], ids=get_parameter_name)
+@pytest.mark.parametrize("provider", [OpenSSL, S2N], ids=get_parameter_name)
+@pytest.mark.parametrize("other_provider", [S2N], ids=get_parameter_name)
+def test_session_resumption_s2n_client_tls13_server_not_tls13(managed_process, cipher, curve, protocol, provider, other_provider, certificate):
+    port = next(available_ports)
+
+    # This test verifies that an S2N client that supports TLS1.3 can resume sessions
+    # with a server that does not support TLS1.3
+
+    # The reconnect option for s2nc allows the client to reconnect automatically
+    # five times. In this test we expect one full connection and five resumption
+    # connections.
+    num_full_connections = 1
+    num_resumed_connections = 5
+
+    client_options = ProviderOptions(
+        mode=Provider.ClientMode,
+        port=port,
+        cipher=cipher,
+        curve=curve,
+        insecure=True,
+        reconnect=True,
+        use_session_ticket=True,
+        protocol=Protocols.TLS13)
+
+    server_options = ProviderOptions(
+        mode=Provider.ServerMode,
+        port=port,
+        cipher=cipher,
+        curve=curve,
+        insecure=True,
+        reconnect=True,
+        use_session_ticket=False,
+        protocol=protocol,
+        reconnects_before_exit=num_resumed_connections + num_full_connections,
+        key=certificate.key,
+        cert=certificate.cert)
+
+    # Passing the type of client and server as a parameter will
+    # allow us to use a fixture to enumerate all possibilities.
+    server = managed_process(provider, server_options, timeout=5)
+    client = managed_process(S2N, client_options, timeout=5)
+
+    expected_version = get_expected_s2n_version(protocol, provider)
+    server_accepts_str = str(
+        num_resumed_connections + num_full_connections) + " server accepts that finished"
+
+    for results in client.get_results():
+        results.assert_success()
+        assert results.stdout.count(
+            b'Resumed session') == num_resumed_connections
+        assert to_bytes("Actual protocol version: {}".format(
+            expected_version)) in results.stdout
+
+    for results in server.get_results():
+        results.assert_success()
+        if provider is S2N:
+            assert results.stdout.count(
+                b'Resumed session') == num_resumed_connections
+            assert to_bytes("Actual protocol version: {}".format(
+                expected_version)) in results.stdout
+        else:
+            assert to_bytes(server_accepts_str) in results.stdout
