@@ -51,9 +51,12 @@ int main(int argc, char **argv)
     uint8_t random_data[S2N_SMALL_FRAGMENT_LENGTH + 1];
     uint8_t aes128_key[] = "123456789012345";
     uint8_t aes256_key[] = "1234567890123456789012345678901";
-    struct s2n_blob aes128 = { .data = aes128_key, .size = sizeof(aes128_key) };
-    struct s2n_blob aes256 = { .data = aes256_key, .size = sizeof(aes256_key) };
-    struct s2n_blob r = { .data = random_data, .size = sizeof(random_data) };
+    struct s2n_blob aes128 = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&aes128, aes128_key, sizeof(aes128_key)));
+    struct s2n_blob aes256 = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&aes256, aes256_key, sizeof(aes256_key)));
+    struct s2n_blob r = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&r, random_data, sizeof(random_data)));
 
     BEGIN_TEST();
     EXPECT_SUCCESS(s2n_disable_tls13_in_test());
@@ -71,7 +74,8 @@ int main(int argc, char **argv)
 
     int max_fragment = S2N_SMALL_FRAGMENT_LENGTH;
     for (size_t i = 0; i <= max_fragment + 1; i++) {
-        struct s2n_blob in = { .data = random_data, .size = i };
+        struct s2n_blob in = { 0 };
+        EXPECT_SUCCESS(s2n_blob_init(&in, random_data, i));
         int bytes_written;
 
         /* TLS packet on the wire using AES-GCM:
@@ -268,7 +272,8 @@ int main(int argc, char **argv)
     conn->actual_protocol_version = S2N_TLS12;
 
     for (size_t i = 0; i <= max_fragment + 1; i++) {
-        struct s2n_blob in = { .data = random_data, .size = i };
+        struct s2n_blob in = { 0 };
+        EXPECT_SUCCESS(s2n_blob_init(&in, random_data, i));
         int bytes_written;
 
         EXPECT_SUCCESS(s2n_connection_wipe(conn));
