@@ -53,9 +53,12 @@ int main(int argc, char **argv)
     uint8_t mac_key_sha256[32] = "server key sha256server key sha";
     uint8_t aes128_key[] = "123456789012345";
     uint8_t aes256_key[] = "1234567890123456789012345678901";
-    struct s2n_blob aes128 = { .data = aes128_key, .size = sizeof(aes128_key) };
-    struct s2n_blob aes256 = { .data = aes256_key, .size = sizeof(aes256_key) };
-    struct s2n_blob r = { .data = random_data, .size = sizeof(random_data) };
+    struct s2n_blob aes128 = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&aes128, aes128_key, sizeof(aes128_key)));
+    struct s2n_blob aes256 = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&aes256, aes256_key, sizeof(aes256_key)));
+    struct s2n_blob r = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&r, random_data, sizeof(random_data)));
     /* Stores explicit IVs used in each test case to validate uniqueness. */
     uint8_t existing_explicit_ivs[S2N_DEFAULT_FRAGMENT_LENGTH + 2][S2N_TLS_MAX_IV_LEN];
 
@@ -87,8 +90,9 @@ int main(int argc, char **argv)
      * There are a few gotchas with respect to explicit IV length and payload length
      */
     for (int j = 0; j < 3; j++) {
-        for (int i = 0; i <= max_aligned_fragment + 1; i++) {
-            struct s2n_blob in = { .data = random_data, .size = i };
+        for (size_t i = 0; i <= max_aligned_fragment + 1; i++) {
+            struct s2n_blob in = { 0 };
+            EXPECT_SUCCESS(s2n_blob_init(&in, random_data, i));
             int bytes_written;
 
             EXPECT_SUCCESS(s2n_connection_wipe(conn));
@@ -163,7 +167,8 @@ int main(int argc, char **argv)
     conn->initial->cipher_suite->record_alg = &s2n_record_alg_aes256_sha_composite;
     for (int j = 0; j < 3; j++) {
         for (int i = 0; i <= max_aligned_fragment + 1; i++) {
-            struct s2n_blob in = { .data = random_data, .size = i };
+            struct s2n_blob in = { 0 };
+            EXPECT_SUCCESS(s2n_blob_init(&in, random_data, i));
             int bytes_written;
 
             EXPECT_SUCCESS(s2n_connection_wipe(conn));
@@ -238,7 +243,8 @@ int main(int argc, char **argv)
     conn->initial->cipher_suite->record_alg = &s2n_record_alg_aes128_sha256_composite;
     for (int j = 0; j < 3; j++) {
         for (int i = 0; i < max_aligned_fragment + 1; i++) {
-            struct s2n_blob in = { .data = random_data, .size = i };
+            struct s2n_blob in = { 0 };
+            EXPECT_SUCCESS(s2n_blob_init(&in, random_data, i));
             int bytes_written;
 
             EXPECT_SUCCESS(s2n_connection_wipe(conn));
@@ -313,7 +319,8 @@ int main(int argc, char **argv)
     conn->initial->cipher_suite->record_alg = &s2n_record_alg_aes256_sha256_composite;
     for (int j = 0; j < 3; j++) {
         for (int i = 0; i <= max_aligned_fragment + 1; i++) {
-            struct s2n_blob in = { .data = random_data, .size = i };
+            struct s2n_blob in = { 0 };
+            EXPECT_SUCCESS(s2n_blob_init(&in, random_data, i));
             int bytes_written;
 
             EXPECT_SUCCESS(s2n_connection_wipe(conn));
