@@ -13,7 +13,7 @@
  * permissions and limitations under the License.
  */
 
-#include "pq-crypto/s2n_pq.h"
+#include "crypto/s2n_pq.h"
 #include "s2n.h"
 #include "s2n_test.h"
 #include "stuffer/s2n_stuffer.h"
@@ -168,24 +168,10 @@ int main(int argc, char **argv)
         };
 
         {
-            const struct s2n_kem_group *test_kem_groups[] = {
-                &s2n_secp256r1_kyber_512_r3,
-#if EVP_APIS_SUPPORTED
-                &s2n_x25519_kyber_512_r3,
-#endif
-            };
-
-            const struct s2n_kem_preferences test_kem_prefs = {
-                .kem_count = 0,
-                .kems = NULL,
-                .tls13_kem_group_count = s2n_array_len(test_kem_groups),
-                .tls13_kem_groups = test_kem_groups,
-            };
-
             const struct s2n_security_policy test_security_policy = {
                 .minimum_protocol_version = S2N_SSLv3,
                 .cipher_preferences = &cipher_preferences_test_all_tls13,
-                .kem_preferences = &test_kem_prefs,
+                .kem_preferences = &kem_preferences_all,
                 .signature_preferences = &s2n_signature_preferences_20200207,
                 .ecc_preferences = &s2n_ecc_preferences_20200310,
             };
@@ -203,7 +189,7 @@ int main(int argc, char **argv)
                 conn->kex_params.server_kem_group_params.kem_group = kem_pref->tls13_kem_groups[0];
                 EXPECT_NULL(conn->kex_params.server_ecc_evp_params.negotiated_curve);
 
-                EXPECT_FAILURE_WITH_ERRNO(s2n_server_hello_retry_recv(conn), S2N_ERR_PQ_DISABLED);
+                EXPECT_FAILURE_WITH_ERRNO(s2n_server_hello_retry_recv(conn), S2N_ERR_NO_SUPPORTED_LIBCRYPTO_API);
 
                 EXPECT_SUCCESS(s2n_connection_free(conn));
             } else {
@@ -218,18 +204,21 @@ int main(int argc, char **argv)
                     POSIX_GUARD(s2n_connection_get_kem_preferences(conn, &kem_pref));
                     EXPECT_NOT_NULL(kem_pref);
 
-                    conn->kex_params.server_kem_group_params.kem_group = kem_pref->tls13_kem_groups[0];
+                    const struct s2n_kem_group *kem_group = s2n_kem_preferences_get_highest_priority_group(kem_pref);
+                    EXPECT_NOT_NULL(kem_group);
+
+                    conn->kex_params.server_kem_group_params.kem_group = kem_group;
                     EXPECT_NULL(conn->kex_params.server_ecc_evp_params.negotiated_curve);
 
                     struct s2n_kem_group_params *client_params = &conn->kex_params.client_kem_group_params;
-                    client_params->kem_group = kem_pref->tls13_kem_groups[0];
-                    client_params->kem_params.kem = kem_pref->tls13_kem_groups[0]->kem;
-                    client_params->ecc_params.negotiated_curve = kem_pref->tls13_kem_groups[0]->curve;
+                    client_params->kem_group = kem_group;
+                    client_params->kem_params.kem = kem_group->kem;
+                    client_params->ecc_params.negotiated_curve = kem_group->curve;
 
                     EXPECT_NULL(client_params->ecc_params.evp_pkey);
                     EXPECT_NULL(client_params->kem_params.private_key.data);
 
-                    kem_public_key_size public_key_size = kem_pref->tls13_kem_groups[0]->kem->public_key_length;
+                    kem_public_key_size public_key_size = kem_group->kem->public_key_length;
                     EXPECT_SUCCESS(s2n_alloc(&client_params->kem_params.public_key, public_key_size));
 
                     EXPECT_OK(s2n_kem_generate_keypair(&client_params->kem_params));
@@ -263,7 +252,9 @@ int main(int argc, char **argv)
                 };
                 /* Test PQ KEM success case for s2n_server_hello_retry_recv. */
                 /* Need at least two KEM's to test fallback */
-                if (test_security_policy.kem_preferences->tls13_kem_group_count >= 2) {
+                uint32_t available_groups = 0;
+                EXPECT_OK(s2n_kem_preferences_groups_available(test_security_policy.kem_preferences, &available_groups));
+                if (available_groups >= 2) {
                     struct s2n_config *config;
                     struct s2n_connection *conn;
 
@@ -800,6 +791,40 @@ int main(int argc, char **argv)
                     S2N_ERR_BAD_MESSAGE);
         };
 
+        /* Test: outside of testing, the server accepts an incorrectly updated ClientHello */
+        {
+            DEFER_CLEANUP(struct s2n_connection *server_conn = s2n_connection_new(S2N_SERVER),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(server_conn);
+            EXPECT_SUCCESS(s2n_connection_set_blinding(server_conn, S2N_SELF_SERVICE_BLINDING));
+            EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
+
+            DEFER_CLEANUP(struct s2n_connection *client_conn = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(client_conn);
+            EXPECT_SUCCESS(s2n_connection_set_blinding(client_conn, S2N_SELF_SERVICE_BLINDING));
+            EXPECT_SUCCESS(s2n_connection_set_config(client_conn, config));
+
+            struct s2n_test_io_pair io_pair = { 0 };
+            EXPECT_SUCCESS(s2n_io_pair_init_non_blocking(&io_pair));
+            EXPECT_SUCCESS(s2n_connections_set_io_pair(client_conn, server_conn, &io_pair));
+
+            /* Force the HRR path */
+            client_conn->security_policy_override = &security_policy_test_tls13_retry;
+
+            /* Send ClientHello */
+            s2n_blocked_status blocked = 0;
+            EXPECT_OK(s2n_negotiate_until_message(client_conn, &blocked, SERVER_HELLO));
+
+            /* Change client random */
+            client_conn->handshake_params.client_random[0]++;
+
+            /* Expect success if we pretend that this isn't a unit test */
+            EXPECT_SUCCESS(s2n_in_unit_test_set(false));
+            EXPECT_SUCCESS(s2n_negotiate_test_server_and_client(server_conn, client_conn));
+            EXPECT_SUCCESS(s2n_in_unit_test_set(true));
+        }
+
         /* Test: The server rejects a second ClientHello with a changed extension */
         {
             DEFER_CLEANUP(struct s2n_connection *server_conn = s2n_connection_new(S2N_SERVER),
