@@ -355,8 +355,6 @@ int main(int argc, char **argv)
             EXPECT_FALSE(IS_HELLO_RETRY_HANDSHAKE(server_conn));
             EXPECT_EQUAL(s2n_conn_get_current_message_type(client_conn), APPLICATION_DATA);
             EXPECT_EQUAL(s2n_conn_get_current_message_type(server_conn), APPLICATION_DATA);
-            EXPECT_FALSE(client_conn->closed);
-            EXPECT_FALSE(server_conn->closed);
 
             EXPECT_SUCCESS(s2n_shutdown_test_server_and_client(server_conn, client_conn));
             EXPECT_SUCCESS(s2n_connection_free(client_conn));
@@ -482,8 +480,6 @@ int main(int argc, char **argv)
                 EXPECT_TRUE(WITH_EARLY_CLIENT_CCS(server_conn));
                 EXPECT_TRUE(IS_HELLO_RETRY_HANDSHAKE(client_conn));
                 EXPECT_TRUE(IS_HELLO_RETRY_HANDSHAKE(server_conn));
-                EXPECT_FALSE(client_conn->closed);
-                EXPECT_FALSE(server_conn->closed);
 
                 EXPECT_SUCCESS(s2n_shutdown_test_server_and_client(server_conn, client_conn));
                 EXPECT_SUCCESS(s2n_connection_wipe(client_conn));
