@@ -25,8 +25,13 @@ int main()
 {
     BEGIN_TEST();
 
-/* Need at least two KEM's to test fallback */
-#if (S2N_SUPPORTED_KEM_GROUPS_COUNT > 1)
+    /* Need at least two KEM's available to test fallback */
+    uint32_t available_groups = 0;
+    EXPECT_OK(s2n_kem_preferences_groups_available(&kem_preferences_all, &available_groups));
+    if (available_groups < 2) {
+        END_TEST();
+        return 0;
+    }
 
     EXPECT_SUCCESS(s2n_enable_tls13_in_test());
 
@@ -109,24 +114,10 @@ int main()
     };
 
     {
-        const struct s2n_kem_group *test_kem_groups[] = {
-            &s2n_secp256r1_kyber_512_r3,
-    #if EVP_APIS_SUPPORTED
-            &s2n_x25519_kyber_512_r3,
-    #endif
-        };
-
-        const struct s2n_kem_preferences test_kem_pref = {
-            .kem_count = 0,
-            .kems = NULL,
-            .tls13_kem_group_count = s2n_array_len(test_kem_groups),
-            .tls13_kem_groups = test_kem_groups,
-        };
-
         const struct s2n_security_policy test_security_policy = {
             .minimum_protocol_version = S2N_SSLv3,
             .cipher_preferences = &cipher_preferences_test_all_tls13,
-            .kem_preferences = &test_kem_pref,
+            .kem_preferences = &kem_preferences_all,
             .signature_preferences = &s2n_signature_preferences_20200207,
             .ecc_preferences = &s2n_ecc_preferences_20200310,
         };
@@ -442,7 +433,6 @@ int main()
             EXPECT_SUCCESS(s2n_connection_free(server_conn));
         };
     };
-#endif
     END_TEST();
     return 0;
 }
