@@ -175,7 +175,7 @@ int main(int argc, char **argv)
     {
         struct s2n_pkey rsa_public_key;
         s2n_pkey_type rsa_pkey_type;
-        EXPECT_SUCCESS(s2n_asn1der_to_public_key_and_type(&rsa_public_key, &rsa_pkey_type, &rsa_cert_chain->cert_chain->head->raw));
+        EXPECT_OK(s2n_asn1der_to_public_key_and_type(&rsa_public_key, &rsa_pkey_type, &rsa_cert_chain->cert_chain->head->raw));
         EXPECT_EQUAL(rsa_pkey_type, S2N_PKEY_TYPE_RSA);
 
         hash_state_new(sign_hash, random_msg);
@@ -185,9 +185,9 @@ int main(int argc, char **argv)
 
         if (!s2n_is_rsa_pss_signing_supported()) {
             EXPECT_FAILURE_WITH_ERRNO(rsa_public_key.sign(rsa_cert_chain->private_key, S2N_SIGNATURE_RSA_PSS_RSAE, &sign_hash, &result),
-                    S2N_RSA_PSS_NOT_SUPPORTED);
+                    S2N_ERR_RSA_PSS_NOT_SUPPORTED);
             EXPECT_FAILURE_WITH_ERRNO(rsa_public_key.verify(&rsa_public_key, S2N_SIGNATURE_RSA_PSS_RSAE, &verify_hash, &result),
-                    S2N_RSA_PSS_NOT_SUPPORTED);
+                    S2N_ERR_RSA_PSS_NOT_SUPPORTED);
         } else {
             EXPECT_SUCCESS(rsa_public_key.sign(rsa_cert_chain->private_key, S2N_SIGNATURE_RSA_PSS_RSAE, &sign_hash, &result));
             EXPECT_SUCCESS(rsa_public_key.verify(&rsa_public_key, S2N_SIGNATURE_RSA_PSS_RSAE, &verify_hash, &result));
@@ -206,7 +206,7 @@ int main(int argc, char **argv)
     {
         struct s2n_pkey rsa_public_key;
         s2n_pkey_type rsa_pkey_type;
-        EXPECT_SUCCESS(s2n_asn1der_to_public_key_and_type(&rsa_public_key, &rsa_pkey_type, &rsa_cert_chain->cert_chain->head->raw));
+        EXPECT_OK(s2n_asn1der_to_public_key_and_type(&rsa_public_key, &rsa_pkey_type, &rsa_cert_chain->cert_chain->head->raw));
         EXPECT_EQUAL(rsa_pkey_type, S2N_PKEY_TYPE_RSA);
 
         /* Test: RSA cert can sign/verify with PSS */
@@ -243,7 +243,7 @@ int main(int argc, char **argv)
         {
             struct s2n_pkey rsa_pss_public_key;
             s2n_pkey_type rsa_pss_pkey_type;
-            EXPECT_SUCCESS(s2n_asn1der_to_public_key_and_type(&rsa_pss_public_key, &rsa_pss_pkey_type, &rsa_pss_cert_chain->cert_chain->head->raw));
+            EXPECT_OK(s2n_asn1der_to_public_key_and_type(&rsa_pss_public_key, &rsa_pss_pkey_type, &rsa_pss_cert_chain->cert_chain->head->raw));
             EXPECT_EQUAL(rsa_pss_pkey_type, S2N_PKEY_TYPE_RSA_PSS);
 
             /* Set the keys equal. */
@@ -281,7 +281,7 @@ int main(int argc, char **argv)
 
         struct s2n_pkey rsa_public_key = { 0 };
         s2n_pkey_type rsa_pkey_type = 0;
-        EXPECT_SUCCESS(s2n_asn1der_to_public_key_and_type(&rsa_public_key, &rsa_pkey_type,
+        EXPECT_OK(s2n_asn1der_to_public_key_and_type(&rsa_public_key, &rsa_pkey_type,
                 &rsa_cert_chain->cert_chain->head->raw));
         EXPECT_EQUAL(rsa_pkey_type, S2N_PKEY_TYPE_RSA);
 
