@@ -37,7 +37,6 @@ struct client_hello_context {
      * this flag tests the previous behavior from blocking callbacks
      */
     int legacy_rc_for_server_name_used;
-    bool mark_done;
 };
 
 int mock_client(struct s2n_test_io_pair *io_pair, int expect_failure, int expect_server_name_used)
@@ -102,7 +101,7 @@ int mock_client(struct s2n_test_io_pair *io_pair, int expect_failure, int expect
     s2n_cleanup();
     s2n_io_pair_close_one_end(io_pair, S2N_CLIENT);
 
-    _exit(result);
+    exit(result);
 }
 
 int client_hello_swap_config(struct s2n_connection *conn, void *ctx)
@@ -183,24 +182,6 @@ int client_hello_fail_handshake(struct s2n_connection *conn, void *ctx)
     return -1;
 }
 
-int s2n_client_hello_poll_cb(struct s2n_connection *conn, void *ctx)
-{
-    struct client_hello_context *client_hello_ctx;
-    if (ctx == NULL) {
-        return -1;
-    }
-    client_hello_ctx = ctx;
-    /* Increment counter to ensure that callback was invoked */
-    client_hello_ctx->invoked++;
-
-    if (client_hello_ctx->mark_done) {
-        EXPECT_SUCCESS(s2n_client_hello_cb_done(conn));
-        return S2N_SUCCESS;
-    }
-
-    return S2N_SUCCESS;
-}
-
 int s2n_negotiate_nonblocking_ch_cb(struct s2n_connection *conn,
         struct client_hello_context *ch_ctx, bool server_name_used)
 {
@@ -229,34 +210,6 @@ int s2n_negotiate_nonblocking_ch_cb(struct s2n_connection *conn,
     return s2n_negotiate(conn, &blocked);
 }
 
-int s2n_negotiate_nonblocking_poll(struct s2n_connection *conn,
-        struct client_hello_context *ch_ctx)
-{
-    EXPECT_NOT_NULL(conn);
-    EXPECT_NOT_NULL(ch_ctx);
-    int invoked = 0;
-    s2n_blocked_status blocked = S2N_NOT_BLOCKED;
-
-    EXPECT_EQUAL(ch_ctx->invoked, 0);
-
-    do {
-        /* negotiate handshake, we should pause after the nonblocking callback is invoked */
-        EXPECT_FAILURE_WITH_ERRNO(s2n_negotiate(conn, &blocked), S2N_ERR_ASYNC_BLOCKED);
-        EXPECT_EQUAL(blocked, S2N_BLOCKED_ON_APPLICATION_INPUT);
-        invoked++;
-        EXPECT_EQUAL(ch_ctx->invoked, invoked);
-    } while (invoked < 10);
-    EXPECT_EQUAL(ch_ctx->invoked, invoked);
-
-    ch_ctx->mark_done = true;
-
-    /* Expect the callback to complete after 2nd iteration */
-    EXPECT_SUCCESS(s2n_negotiate(conn, &blocked));
-    EXPECT_EQUAL(ch_ctx->invoked, invoked + 1);
-
-    return S2N_SUCCESS;
-}
-
 int s2n_negotiate_blocking_ch_cb(struct s2n_connection *conn, struct client_hello_context *ch_ctx)
 {
     s2n_blocked_status blocked;
@@ -325,35 +278,40 @@ int start_client_conn(struct s2n_test_io_pair *io_pair, pid_t *pid,
 
 static int test_case_clean(struct s2n_connection *conn, pid_t client_pid,
         struct s2n_config *config, struct s2n_test_io_pair *io_pair,
-        struct client_hello_context *ch_ctx)
+        struct client_hello_context *ch_ctx, struct s2n_cert_chain_and_key *chain_and_key)
 {
     s2n_blocked_status blocked;
     int status;
 
-    if (conn != NULL) {
-        EXPECT_SUCCESS(s2n_shutdown(conn, &blocked));
-        EXPECT_SUCCESS(s2n_connection_free(conn));
-    }
+    EXPECT_SUCCESS(s2n_shutdown(conn, &blocked));
     EXPECT_EQUAL(waitpid(-1, &status, 0), client_pid);
     EXPECT_EQUAL(status, 0);
-    EXPECT_SUCCESS(s2n_config_free(config));
     /* client process cleans their end, we just need to close server side */
     EXPECT_SUCCESS(s2n_io_pair_close_one_end(io_pair, S2N_SERVER));
+
+    EXPECT_SUCCESS(s2n_connection_free(conn));
+    EXPECT_SUCCESS(s2n_config_free(config));
     memset(ch_ctx, 0, sizeof(struct client_hello_context));
+    EXPECT_SUCCESS(s2n_cert_chain_and_key_free(chain_and_key));
+
     return S2N_SUCCESS;
 }
 
 int run_test_config_swap_ch_cb(s2n_client_hello_cb_mode cb_mode,
-        struct s2n_cert_chain_and_key *chain_and_key, struct client_hello_context *ch_ctx)
+        struct client_hello_context *ch_ctx)
 {
     struct s2n_test_io_pair io_pair;
     struct s2n_config *config;
     struct s2n_connection *conn;
     struct s2n_config *swap_config;
     pid_t pid;
+    struct s2n_cert_chain_and_key *chain_and_key;
+
+    EXPECT_SUCCESS(start_client_conn(&io_pair, &pid, 0, 1));
 
     /* Add application protocols to swapped config */
     static const char *protocols[] = { "h2" };
+    EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&chain_and_key, S2N_DEFAULT_TEST_CERT_CHAIN, S2N_DEFAULT_TEST_PRIVATE_KEY));
 
     /* prepare swap_config */
     EXPECT_NOT_NULL(swap_config = s2n_config_new());
@@ -371,7 +329,6 @@ int run_test_config_swap_ch_cb(s2n_client_hello_cb_mode cb_mode,
     EXPECT_SUCCESS(s2n_config_set_client_hello_cb_mode(config, cb_mode));
     EXPECT_SUCCESS(s2n_config_set_client_hello_cb(config, client_hello_swap_config, ch_ctx));
 
-    EXPECT_SUCCESS(start_client_conn(&io_pair, &pid, 0, 1));
     EXPECT_SUCCESS(init_server_conn(&conn, &io_pair, config));
 
     /* do the handshake */
@@ -394,27 +351,28 @@ int run_test_config_swap_ch_cb(s2n_client_hello_cb_mode cb_mode,
 
     EXPECT_SUCCESS(server_recv(conn));
 
-    EXPECT_SUCCESS(test_case_clean(conn, pid, config, &io_pair, ch_ctx));
+    EXPECT_SUCCESS(test_case_clean(conn, pid, config, &io_pair, ch_ctx, chain_and_key));
     EXPECT_SUCCESS(s2n_config_free(swap_config));
     return S2N_SUCCESS;
 }
 
-int run_test_no_config_swap_ch_cb(s2n_client_hello_cb_mode cb_mode,
-        struct s2n_cert_chain_and_key *chain_and_key,
-        struct client_hello_context *ch_ctx)
+int run_test_no_config_swap_ch_cb(s2n_client_hello_cb_mode cb_mode, struct client_hello_context *ch_ctx)
 {
     struct s2n_test_io_pair io_pair;
     struct s2n_config *config;
     struct s2n_connection *conn;
     pid_t pid;
+    struct s2n_cert_chain_and_key *chain_and_key;
 
+    EXPECT_SUCCESS(start_client_conn(&io_pair, &pid, 0, 0));
+
+    EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&chain_and_key, S2N_DEFAULT_TEST_CERT_CHAIN, S2N_DEFAULT_TEST_PRIVATE_KEY));
     EXPECT_NOT_NULL(config = s2n_config_new());
     EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key_to_store(config, chain_and_key));
 
     /* Setup ClientHello callback */
     EXPECT_SUCCESS(s2n_config_set_client_hello_cb(config, client_hello_swap_config, ch_ctx));
     EXPECT_SUCCESS(s2n_config_set_client_hello_cb_mode(config, cb_mode));
-    EXPECT_SUCCESS(start_client_conn(&io_pair, &pid, 0, 0));
     EXPECT_SUCCESS(init_server_conn(&conn, &io_pair, config));
 
     /* do the handshake */
@@ -431,20 +389,22 @@ int run_test_no_config_swap_ch_cb(s2n_client_hello_cb_mode cb_mode,
 
     EXPECT_SUCCESS(server_recv(conn));
 
-    EXPECT_SUCCESS(test_case_clean(conn, pid, config, &io_pair, ch_ctx));
+    EXPECT_SUCCESS(test_case_clean(conn, pid, config, &io_pair, ch_ctx, chain_and_key));
     return S2N_SUCCESS;
 }
 
-int run_test_reject_handshake_ch_cb(s2n_client_hello_cb_mode cb_mode,
-        struct s2n_cert_chain_and_key *chain_and_key,
-        struct client_hello_context *ch_ctx)
+int run_test_reject_handshake_ch_cb(s2n_client_hello_cb_mode cb_mode, struct client_hello_context *ch_ctx)
 {
     struct s2n_test_io_pair io_pair;
     struct s2n_config *config;
     struct s2n_connection *conn;
     pid_t pid;
     s2n_blocked_status blocked;
+    struct s2n_cert_chain_and_key *chain_and_key;
 
+    EXPECT_SUCCESS(start_client_conn(&io_pair, &pid, 1, 0));
+
+    EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&chain_and_key, S2N_DEFAULT_TEST_CERT_CHAIN, S2N_DEFAULT_TEST_PRIVATE_KEY));
     EXPECT_NOT_NULL(config = s2n_config_new());
     EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key_to_store(config, chain_and_key));
 
@@ -452,8 +412,6 @@ int run_test_reject_handshake_ch_cb(s2n_client_hello_cb_mode cb_mode,
     EXPECT_SUCCESS(s2n_config_set_client_hello_cb(config, client_hello_fail_handshake, ch_ctx));
     EXPECT_SUCCESS(s2n_config_set_client_hello_cb_mode(config, cb_mode));
 
-    EXPECT_SUCCESS(start_client_conn(&io_pair, &pid, 1, 0));
-
     EXPECT_SUCCESS(init_server_conn(&conn, &io_pair, config));
     /* If s2n_negotiate fails, it usually would delay with a sleep. In order to
      * test that we don't blind when CLientHello callback fails the handshake,
@@ -469,104 +427,45 @@ int run_test_reject_handshake_ch_cb(s2n_client_hello_cb_mode cb_mode,
     /* Ensure that callback was invoked */
     EXPECT_EQUAL(ch_ctx->invoked, 1);
 
-    /* shutdown to flush alert, expext failure as client doesn't send close notify */
-    EXPECT_FAILURE(s2n_shutdown(conn, &blocked));
-    EXPECT_SUCCESS(s2n_connection_free(conn));
-
-    EXPECT_SUCCESS(test_case_clean(NULL, pid, config, &io_pair, ch_ctx));
-    return S2N_SUCCESS;
-}
-
-int run_test_poll_ch_cb(s2n_client_hello_cb_mode cb_mode,
-        struct s2n_cert_chain_and_key *chain_and_key,
-        struct client_hello_context *ch_ctx)
-{
-    struct s2n_test_io_pair io_pair = { 0 };
-    struct s2n_config *config = s2n_config_new();
-    EXPECT_NOT_NULL(config);
-    struct s2n_connection *conn;
-    pid_t pid = 0;
-
-    EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key_to_store(config, chain_and_key));
-
-    /* Setup ClientHello callback */
-    EXPECT_SUCCESS(s2n_config_set_client_hello_cb(config, s2n_client_hello_poll_cb, ch_ctx));
-    EXPECT_SUCCESS(s2n_config_set_client_hello_cb_mode(config, cb_mode));
-
-    /* Enable callback polling mode */
-    EXPECT_SUCCESS(s2n_config_client_hello_cb_enable_poll(config));
-
-    EXPECT_SUCCESS(start_client_conn(&io_pair, &pid, 0, 0));
-    EXPECT_SUCCESS(init_server_conn(&conn, &io_pair, config));
-
-    /* negotiate and make assertions */
-    EXPECT_SUCCESS(s2n_negotiate_nonblocking_poll(conn, ch_ctx));
-
-    EXPECT_SUCCESS(server_recv(conn));
-
-    EXPECT_SUCCESS(test_case_clean(conn, pid, config, &io_pair, ch_ctx));
+    /* shutdown to flush alert */
+    EXPECT_SUCCESS(test_case_clean(conn, pid, config, &io_pair, ch_ctx, chain_and_key));
     return S2N_SUCCESS;
 }
 
 int main(int argc, char **argv)
 {
     struct client_hello_context client_hello_ctx = { 0 };
-    char *cert_chain_pem;
-    char *private_key_pem;
-    struct s2n_cert_chain_and_key *chain_and_key;
     BEGIN_TEST();
-
-    EXPECT_NOT_NULL(cert_chain_pem = malloc(S2N_MAX_TEST_PEM_SIZE));
-    EXPECT_NOT_NULL(private_key_pem = malloc(S2N_MAX_TEST_PEM_SIZE));
-    EXPECT_SUCCESS(s2n_read_test_pem(S2N_DEFAULT_TEST_CERT_CHAIN, cert_chain_pem, S2N_MAX_TEST_PEM_SIZE));
-    EXPECT_SUCCESS(s2n_read_test_pem(S2N_DEFAULT_TEST_PRIVATE_KEY, private_key_pem, S2N_MAX_TEST_PEM_SIZE));
-    EXPECT_NOT_NULL(chain_and_key = s2n_cert_chain_and_key_new());
-    EXPECT_SUCCESS(s2n_cert_chain_and_key_load_pem(chain_and_key, cert_chain_pem, private_key_pem));
-    /** Test config swapping in client hello callback **/
+    /* Test config swapping in client hello callback */
 
     /* we want to update the config outside of callback so don't swap in callback */
     client_hello_ctx.swap_config_nonblocking_mode = 1;
-    EXPECT_SUCCESS(run_test_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_NONBLOCKING,
-            chain_and_key, &client_hello_ctx));
+    EXPECT_SUCCESS(run_test_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_NONBLOCKING, &client_hello_ctx));
 
     /* non blocking callback when callback marks cb_done during the callback */
     client_hello_ctx.swap_config_during_callback = 1;
     client_hello_ctx.mark_done_during_callback = 1;
-    EXPECT_SUCCESS(run_test_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_NONBLOCKING,
-            chain_and_key, &client_hello_ctx));
+    EXPECT_SUCCESS(run_test_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_NONBLOCKING, &client_hello_ctx));
 
     /* we want to update the config in the callback */
     client_hello_ctx.swap_config_during_callback = 1;
-    EXPECT_SUCCESS(run_test_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_BLOCKING,
-            chain_and_key, &client_hello_ctx));
+    EXPECT_SUCCESS(run_test_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_BLOCKING, &client_hello_ctx));
 
     /* validate legacy behavior for server_name_used */
     /* we want to update the config in the callback */
     client_hello_ctx.swap_config_during_callback = 1;
     client_hello_ctx.legacy_rc_for_server_name_used = 1;
-    EXPECT_SUCCESS(run_test_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_BLOCKING,
-            chain_and_key, &client_hello_ctx));
+    EXPECT_SUCCESS(run_test_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_BLOCKING, &client_hello_ctx));
 
-    /** Tests for test when server_name_used is not set **/
-    EXPECT_SUCCESS(run_test_no_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_BLOCKING,
-            chain_and_key, &client_hello_ctx));
+    /* Tests for test when server_name_used is not set */
+    EXPECT_SUCCESS(run_test_no_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_BLOCKING, &client_hello_ctx));
 
-    EXPECT_SUCCESS(run_test_no_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_NONBLOCKING,
-            chain_and_key, &client_hello_ctx));
+    EXPECT_SUCCESS(run_test_no_config_swap_ch_cb(S2N_CLIENT_HELLO_CB_NONBLOCKING, &client_hello_ctx));
 
-    /** Test rejecting connection in client hello callback **/
-    EXPECT_SUCCESS(run_test_reject_handshake_ch_cb(S2N_CLIENT_HELLO_CB_BLOCKING,
-            chain_and_key, &client_hello_ctx));
+    /* Test rejecting connection in client hello callback */
+    EXPECT_SUCCESS(run_test_reject_handshake_ch_cb(S2N_CLIENT_HELLO_CB_BLOCKING, &client_hello_ctx));
 
-    EXPECT_SUCCESS(run_test_reject_handshake_ch_cb(S2N_CLIENT_HELLO_CB_NONBLOCKING,
-            chain_and_key, &client_hello_ctx));
-
-    EXPECT_SUCCESS(run_test_poll_ch_cb(S2N_CLIENT_HELLO_CB_NONBLOCKING,
-            chain_and_key, &client_hello_ctx));
-
-    EXPECT_SUCCESS(s2n_cert_chain_and_key_free(chain_and_key));
-    free(cert_chain_pem);
-    free(private_key_pem);
+    EXPECT_SUCCESS(run_test_reject_handshake_ch_cb(S2N_CLIENT_HELLO_CB_NONBLOCKING, &client_hello_ctx));
 
     END_TEST();
 
