@@ -22,7 +22,7 @@
 int main(int argc, char **argv)
 {
     uint8_t entropy[2048] = { 0 };
-    struct s2n_stuffer stuffer;
+    struct s2n_stuffer stuffer = { 0 };
     uint8_t u8;
     uint16_t u16;
     uint32_t u32;
@@ -35,7 +35,8 @@ int main(int argc, char **argv)
     EXPECT_SUCCESS(s2n_stuffer_alloc(&stuffer, 100));
 
     /* Try to write 101 bytes */
-    struct s2n_blob in = { .data = entropy, .size = 101 };
+    struct s2n_blob in = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&in, entropy, 101));
     EXPECT_FAILURE(s2n_stuffer_write(&stuffer, &in));
 
     /* Try to write 101 1-byte ints bytes */
@@ -139,34 +140,38 @@ int main(int argc, char **argv)
 
 #ifndef NDEBUG
     /* Invalid blob should fail init */
-    struct s2n_stuffer s1;
+    struct s2n_stuffer s1 = { 0 };
     struct s2n_blob b1 = { .data = 0, .size = 101 };
     EXPECT_FAILURE(s2n_stuffer_init(&s1, &b1));
 #endif
 
     /* Valid empty blob should succeed init */
-    struct s2n_stuffer s2;
-    struct s2n_blob b2 = { .data = 0, .size = 0 };
+    struct s2n_stuffer s2 = { 0 };
+    struct s2n_blob b2 = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&b2, 0, 0));
     EXPECT_SUCCESS(s2n_stuffer_init(&s2, &b2));
 
     /* Valid blob should succeed init */
-    struct s2n_stuffer s3;
+    struct s2n_stuffer s3 = { 0 };
     uint8_t a3[12];
-    struct s2n_blob b3 = { .data = a3, .size = sizeof(a3) };
+    struct s2n_blob b3 = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&b3, a3, sizeof(a3)));
     EXPECT_SUCCESS(s2n_stuffer_init(&s3, &b3));
 
     /* Null blob should fail init */
-    struct s2n_stuffer s4;
+    struct s2n_stuffer s4 = { 0 };
     EXPECT_FAILURE(s2n_stuffer_init(&s4, NULL));
 
     /* Null stuffer should fail init */
-    struct s2n_blob b5 = { .data = 0, .size = 0 };
+    struct s2n_blob b5 = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&b5, 0, 0));
     EXPECT_FAILURE(s2n_stuffer_init(NULL, &b5));
 
     /* Check s2n_stuffer_validate() function */
     EXPECT_ERROR(s2n_stuffer_validate(NULL));
     uint8_t valid_blob_array[12];
-    struct s2n_blob blob_valid = { .data = valid_blob_array, .size = sizeof(valid_blob_array) };
+    struct s2n_blob blob_valid = { 0 };
+    EXPECT_SUCCESS(s2n_blob_init(&blob_valid, valid_blob_array, sizeof(valid_blob_array)));
 
     struct s2n_stuffer stuffer_valid = { 0 };
     EXPECT_SUCCESS(s2n_stuffer_init(&stuffer_valid, &blob_valid));
@@ -204,5 +209,36 @@ int main(int argc, char **argv)
     EXPECT_EQUAL(s2n_stuffer_data_available(&reserve_test_stuffer), 0);
     EXPECT_SUCCESS(s2n_stuffer_free(&reserve_test_stuffer));
 
+    /* Test: s2n_stuffer_init_written */
+    {
+        uint8_t data[] = "hello world";
+        uint8_t input[sizeof(data)] = { 0 };
+        uint8_t output[sizeof(data)] = { 0 };
+
+        struct s2n_blob blob = { 0 };
+        EXPECT_SUCCESS(s2n_blob_init(&blob, input, sizeof(input)));
+
+        /* Repeat control to show behavior is consistent */
+        for (size_t i = 0; i < 3; i++) {
+            struct s2n_stuffer unwritten_stuffer = { 0 };
+            EXPECT_SUCCESS(s2n_stuffer_init(&unwritten_stuffer, &blob));
+            EXPECT_EQUAL(s2n_stuffer_data_available(&unwritten_stuffer), 0);
+            EXPECT_FAILURE_WITH_ERRNO(s2n_stuffer_read_bytes(&unwritten_stuffer, output, sizeof(output)),
+                    S2N_ERR_STUFFER_OUT_OF_DATA);
+            EXPECT_SUCCESS(s2n_stuffer_write_bytes(&unwritten_stuffer, data, sizeof(data)));
+        }
+
+        /* Repeat test to show behavior is consistent */
+        for (size_t i = 0; i < 3; i++) {
+            struct s2n_stuffer written_stuffer = { 0 };
+            EXPECT_SUCCESS(s2n_stuffer_init_written(&written_stuffer, &blob));
+            EXPECT_EQUAL(s2n_stuffer_data_available(&written_stuffer), sizeof(data));
+            EXPECT_SUCCESS(s2n_stuffer_read_bytes(&written_stuffer, output, sizeof(output)));
+            EXPECT_BYTEARRAY_EQUAL(data, output, sizeof(output));
+            EXPECT_FAILURE_WITH_ERRNO(s2n_stuffer_write_bytes(&written_stuffer, data, sizeof(data)),
+                    S2N_ERR_STUFFER_IS_FULL);
+        }
+    };
+
     END_TEST();
 }
