@@ -107,11 +107,11 @@ int main(int argc, char **argv)
             DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_SERVER),
                     s2n_connection_ptr_free);
             conn->secure->cipher_suite = &s2n_tls13_aes_256_gcm_sha384;
-            conn->secrets.tls13.extract_secret_type = S2N_EARLY_SECRET;
+            conn->secrets.extract_secret_type = S2N_EARLY_SECRET;
 
             EXPECT_OK(s2n_tls13_extract_secret(conn, S2N_EARLY_SECRET));
-            EXPECT_EQUAL(conn->secrets.tls13.extract_secret_type, S2N_EARLY_SECRET);
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.extract_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_EQUAL(conn->secrets.extract_secret_type, S2N_EARLY_SECRET);
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.extract_secret, empty_secret, sizeof(empty_secret));
         };
 
         /* Generate all secrets sequentially  */
@@ -119,20 +119,20 @@ int main(int argc, char **argv)
             DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_SERVER),
                     s2n_connection_ptr_free);
             conn->secure->cipher_suite = &s2n_tls13_aes_128_gcm_sha256;
-            conn->secrets.tls13.extract_secret_type = S2N_NONE_SECRET;
+            conn->secrets.extract_secret_type = S2N_NONE_SECRET;
 
             EXPECT_OK(s2n_tls13_extract_secret(conn, S2N_EARLY_SECRET));
-            EXPECT_EQUAL(conn->secrets.tls13.extract_secret_type, S2N_EARLY_SECRET);
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.extract_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_EQUAL(conn->secrets.extract_secret_type, S2N_EARLY_SECRET);
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.extract_secret, empty_secret, sizeof(empty_secret));
 
             EXPECT_OK(s2n_set_test_key_shares(conn, &s2n_ecc_curve_secp256r1));
             EXPECT_OK(s2n_tls13_extract_secret(conn, S2N_HANDSHAKE_SECRET));
-            EXPECT_EQUAL(conn->secrets.tls13.extract_secret_type, S2N_HANDSHAKE_SECRET);
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.extract_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_EQUAL(conn->secrets.extract_secret_type, S2N_HANDSHAKE_SECRET);
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.extract_secret, empty_secret, sizeof(empty_secret));
 
             EXPECT_OK(s2n_tls13_extract_secret(conn, S2N_MASTER_SECRET));
-            EXPECT_EQUAL(conn->secrets.tls13.extract_secret_type, S2N_MASTER_SECRET);
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.extract_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_EQUAL(conn->secrets.extract_secret_type, S2N_MASTER_SECRET);
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.extract_secret, empty_secret, sizeof(empty_secret));
         };
 
         /* Generate all secrets at once (backfill) */
@@ -140,12 +140,12 @@ int main(int argc, char **argv)
             DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_SERVER),
                     s2n_connection_ptr_free);
             conn->secure->cipher_suite = &s2n_tls13_aes_256_gcm_sha384;
-            conn->secrets.tls13.extract_secret_type = S2N_NONE_SECRET;
+            conn->secrets.extract_secret_type = S2N_NONE_SECRET;
             EXPECT_OK(s2n_set_test_key_shares(conn, &s2n_ecc_curve_secp256r1));
 
             EXPECT_OK(s2n_tls13_extract_secret(conn, S2N_MASTER_SECRET));
-            EXPECT_EQUAL(conn->secrets.tls13.extract_secret_type, S2N_MASTER_SECRET);
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.extract_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_EQUAL(conn->secrets.extract_secret_type, S2N_MASTER_SECRET);
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.extract_secret, empty_secret, sizeof(empty_secret));
         }
 
         /* All valid parameter combinations should succeed */
@@ -153,7 +153,7 @@ int main(int argc, char **argv)
             DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(test_cases[i].conn_mode),
                     s2n_connection_ptr_free);
             conn->secure->cipher_suite = test_cases[i].cipher_suite;
-            conn->secrets.tls13.extract_secret_type = test_cases[i].curr_secret_type;
+            conn->secrets.extract_secret_type = test_cases[i].curr_secret_type;
             EXPECT_OK(s2n_set_test_key_shares(conn, test_cases[i].curve));
             EXPECT_OK(s2n_tls13_extract_secret(conn, test_cases[i].next_secret_type));
         }
@@ -191,7 +191,7 @@ int main(int argc, char **argv)
             DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_SERVER),
                     s2n_connection_ptr_free);
             conn->secure->cipher_suite = &s2n_tls13_aes_256_gcm_sha384;
-            conn->secrets.tls13.extract_secret_type = S2N_NONE_SECRET;
+            conn->secrets.extract_secret_type = S2N_NONE_SECRET;
 
             EXPECT_OK(s2n_tls13_derive_secret(conn, S2N_EARLY_SECRET, S2N_SERVER, &output));
             EXPECT_BYTEARRAY_NOT_EQUAL(output.data, empty_secret, sizeof(empty_secret));
@@ -206,6 +206,7 @@ int main(int argc, char **argv)
             DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_SERVER),
                     s2n_connection_ptr_free);
             conn->actual_protocol_version = S2N_TLS13;
+            EXPECT_OK(s2n_conn_choose_state_machine(conn, S2N_TLS13));
             conn->secure->cipher_suite = &s2n_tls13_aes_256_gcm_sha384;
             conn->handshake.handshake_type = handshake_type;
             EXPECT_OK(s2n_set_test_key_shares(conn, &s2n_ecc_curve_secp256r1));
@@ -229,15 +230,16 @@ int main(int argc, char **argv)
             DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_SERVER),
                     s2n_connection_ptr_free);
             conn->secure->cipher_suite = &s2n_tls13_aes_256_gcm_sha384;
+            EXPECT_OK(s2n_conn_choose_state_machine(conn, S2N_TLS13));
             conn->actual_protocol_version = S2N_TLS13;
             conn->handshake.handshake_type = handshake_type;
             conn->handshake.message_number = message_nums[S2N_HANDSHAKE_SECRET];
             EXPECT_OK(s2n_set_test_key_shares(conn, &s2n_ecc_curve_secp256r1));
 
-            conn->secrets.tls13.extract_secret_type = S2N_NONE_SECRET;
+            conn->secrets.extract_secret_type = S2N_NONE_SECRET;
             EXPECT_OK(s2n_tls13_derive_secret(conn, S2N_HANDSHAKE_SECRET, S2N_SERVER, &output));
-            EXPECT_EQUAL(conn->secrets.tls13.extract_secret_type, S2N_HANDSHAKE_SECRET);
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.extract_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_EQUAL(conn->secrets.extract_secret_type, S2N_HANDSHAKE_SECRET);
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.extract_secret, empty_secret, sizeof(empty_secret));
             EXPECT_BYTEARRAY_NOT_EQUAL(output.data, empty_secret, sizeof(empty_secret));
         };
 
@@ -250,7 +252,8 @@ int main(int argc, char **argv)
             DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(test_cases[i].conn_mode),
                     s2n_connection_ptr_free);
             conn->secure->cipher_suite = test_cases[i].cipher_suite;
-            conn->secrets.tls13.extract_secret_type = test_cases[i].curr_secret_type;
+            conn->secrets.extract_secret_type = test_cases[i].curr_secret_type;
+            EXPECT_OK(s2n_conn_choose_state_machine(conn, S2N_TLS13));
             conn->actual_protocol_version = S2N_TLS13;
             conn->handshake.handshake_type = handshake_type;
             conn->handshake.message_number = message_nums[test_cases[i].next_secret_type];
@@ -272,31 +275,34 @@ int main(int argc, char **argv)
             conn->secure->cipher_suite = &s2n_tls13_aes_128_gcm_sha256;
             conn->actual_protocol_version = S2N_TLS13;
 
-            EXPECT_MEMCPY_SUCCESS(conn->secrets.tls13.extract_secret, test_secret.data, test_secret.size);
-            EXPECT_MEMCPY_SUCCESS(conn->secrets.tls13.client_early_secret, test_secret.data, test_secret.size);
-            EXPECT_MEMCPY_SUCCESS(conn->secrets.tls13.client_handshake_secret, test_secret.data, test_secret.size);
-            EXPECT_MEMCPY_SUCCESS(conn->secrets.tls13.server_handshake_secret, test_secret.data, test_secret.size);
-            EXPECT_MEMCPY_SUCCESS(conn->secrets.tls13.client_app_secret, test_secret.data, test_secret.size);
-            EXPECT_MEMCPY_SUCCESS(conn->secrets.tls13.server_app_secret, test_secret.data, test_secret.size);
-            EXPECT_MEMCPY_SUCCESS(conn->secrets.tls13.resumption_master_secret, test_secret.data, test_secret.size);
+            EXPECT_MEMCPY_SUCCESS(conn->secrets.version.tls13.extract_secret, test_secret.data, test_secret.size);
+            EXPECT_MEMCPY_SUCCESS(conn->secrets.version.tls13.client_early_secret, test_secret.data, test_secret.size);
+            EXPECT_MEMCPY_SUCCESS(conn->secrets.version.tls13.client_handshake_secret, test_secret.data, test_secret.size);
+            EXPECT_MEMCPY_SUCCESS(conn->secrets.version.tls13.server_handshake_secret, test_secret.data, test_secret.size);
+            EXPECT_MEMCPY_SUCCESS(conn->secrets.version.tls13.client_app_secret, test_secret.data, test_secret.size);
+            EXPECT_MEMCPY_SUCCESS(conn->secrets.version.tls13.server_app_secret, test_secret.data, test_secret.size);
+            EXPECT_MEMCPY_SUCCESS(conn->secrets.version.tls13.exporter_master_secret, test_secret.data, test_secret.size);
+            EXPECT_MEMCPY_SUCCESS(conn->secrets.version.tls13.resumption_master_secret, test_secret.data, test_secret.size);
 
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.extract_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.client_early_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.client_handshake_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.server_handshake_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.client_app_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.server_app_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.resumption_master_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.extract_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.client_early_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.client_handshake_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.server_handshake_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.client_app_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.server_app_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.exporter_master_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.resumption_master_secret, empty_secret, sizeof(empty_secret));
 
             EXPECT_OK(s2n_tls13_secrets_clean(conn));
 
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.extract_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.client_early_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.client_handshake_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.server_handshake_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.client_app_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.server_app_secret, empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.resumption_master_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.extract_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.client_early_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.client_handshake_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.server_handshake_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.client_app_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.server_app_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.exporter_master_secret, empty_secret, sizeof(empty_secret));
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.resumption_master_secret, empty_secret, sizeof(empty_secret));
         };
     };
 
@@ -313,7 +319,7 @@ int main(int argc, char **argv)
             EXPECT_ERROR_WITH_ERRNO(s2n_tls13_secrets_get(conn, 100, S2N_CLIENT, &result), S2N_ERR_SAFETY);
             EXPECT_ERROR_WITH_ERRNO(s2n_tls13_secrets_get(conn, S2N_EARLY_SECRET, S2N_SERVER, &result), S2N_ERR_SAFETY);
 
-            conn->secrets.tls13.extract_secret_type = S2N_NONE_SECRET;
+            conn->secrets.extract_secret_type = S2N_NONE_SECRET;
             EXPECT_ERROR_WITH_ERRNO(s2n_tls13_secrets_get(conn, S2N_HANDSHAKE_SECRET, S2N_CLIENT, &result), S2N_ERR_SAFETY);
 
             struct s2n_crypto_parameters *secure = conn->secure;
@@ -334,9 +340,9 @@ int main(int argc, char **argv)
             conn->secure->cipher_suite = &s2n_tls13_aes_128_gcm_sha256;
             conn->actual_protocol_version = S2N_TLS13;
 
-            EXPECT_MEMCPY_SUCCESS(conn->secrets.tls13.client_handshake_secret,
+            EXPECT_MEMCPY_SUCCESS(conn->secrets.version.tls13.client_handshake_secret,
                     test_secret.data, test_secret.size);
-            conn->secrets.tls13.extract_secret_type = S2N_HANDSHAKE_SECRET;
+            conn->secrets.extract_secret_type = S2N_HANDSHAKE_SECRET;
 
             struct s2n_blob result = { 0 };
             uint8_t result_bytes[S2N_TLS13_SECRET_MAX_LEN] = { 0 };
@@ -360,35 +366,35 @@ int main(int argc, char **argv)
                     s2n_connection_ptr_free);
             conn->secure->cipher_suite = &s2n_tls13_aes_128_gcm_sha256;
             conn->actual_protocol_version = S2N_TLS13;
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.client_early_secret,
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.client_early_secret,
                     empty_secret, sizeof(empty_secret));
 
             /* Early secret not derived if early data not requested */
             conn->early_data_state = S2N_EARLY_DATA_NOT_REQUESTED;
             EXPECT_OK(s2n_tls13_secrets_update(conn));
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.client_early_secret,
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.client_early_secret,
                     empty_secret, sizeof(empty_secret));
 
             /* Early secret not derived if early data rejected */
             conn->early_data_state = S2N_EARLY_DATA_REJECTED;
             EXPECT_OK(s2n_tls13_secrets_update(conn));
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.client_early_secret,
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.client_early_secret,
                     empty_secret, sizeof(empty_secret));
 
             /* Early secret derived if early data requested */
             conn->early_data_state = S2N_EARLY_DATA_REQUESTED;
             EXPECT_OK(s2n_tls13_secrets_update(conn));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.client_early_secret,
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.client_early_secret,
                     empty_secret, sizeof(empty_secret));
 
             /* Clear secret */
-            EXPECT_MEMCPY_SUCCESS(conn->secrets.tls13.client_early_secret,
+            EXPECT_MEMCPY_SUCCESS(conn->secrets.version.tls13.client_early_secret,
                     empty_secret, sizeof(empty_secret));
 
             /* Early secret derived if early data accepted */
             conn->early_data_state = S2N_EARLY_DATA_ACCEPTED;
             EXPECT_OK(s2n_tls13_secrets_update(conn));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.client_early_secret,
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.client_early_secret,
                     empty_secret, sizeof(empty_secret));
         };
 
@@ -399,9 +405,9 @@ int main(int argc, char **argv)
             conn->secure->cipher_suite = &s2n_tls13_aes_128_gcm_sha256;
             conn->actual_protocol_version = S2N_TLS13;
             EXPECT_OK(s2n_connection_set_test_handshake_secret(conn, &test_secret));
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.client_handshake_secret,
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.client_handshake_secret,
                     empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.server_handshake_secret,
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.server_handshake_secret,
                     empty_secret, sizeof(empty_secret));
 
             while (s2n_conn_get_current_message_type(conn) != SERVER_HELLO) {
@@ -409,9 +415,9 @@ int main(int argc, char **argv)
             }
             EXPECT_OK(s2n_tls13_secrets_update(conn));
 
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.client_handshake_secret,
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.client_handshake_secret,
                     empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.server_handshake_secret,
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.server_handshake_secret,
                     empty_secret, sizeof(empty_secret));
         };
 
@@ -449,9 +455,9 @@ int main(int argc, char **argv)
             conn->secure->cipher_suite = &s2n_tls13_aes_128_gcm_sha256;
             conn->actual_protocol_version = S2N_TLS13;
             EXPECT_OK(s2n_connection_set_test_master_secret(conn, &test_secret));
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.client_app_secret,
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.client_app_secret,
                     empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.server_app_secret,
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.server_app_secret,
                     empty_secret, sizeof(empty_secret));
 
             while (s2n_conn_get_current_message_type(conn) != SERVER_FINISHED) {
@@ -459,9 +465,9 @@ int main(int argc, char **argv)
             }
             EXPECT_OK(s2n_tls13_secrets_update(conn));
 
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.client_app_secret,
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.client_app_secret,
                     empty_secret, sizeof(empty_secret));
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.server_app_secret,
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.server_app_secret,
                     empty_secret, sizeof(empty_secret));
         };
 
@@ -472,7 +478,7 @@ int main(int argc, char **argv)
             conn->secure->cipher_suite = &s2n_tls13_aes_128_gcm_sha256;
             conn->actual_protocol_version = S2N_TLS13;
             EXPECT_OK(s2n_connection_set_test_master_secret(conn, &test_secret));
-            EXPECT_BYTEARRAY_EQUAL(conn->secrets.tls13.resumption_master_secret,
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.resumption_master_secret,
                     empty_secret, sizeof(empty_secret));
 
             while (s2n_conn_get_current_message_type(conn) != CLIENT_FINISHED) {
@@ -480,10 +486,63 @@ int main(int argc, char **argv)
             }
             EXPECT_OK(s2n_tls13_secrets_update(conn));
 
-            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.tls13.resumption_master_secret,
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.resumption_master_secret,
                     empty_secret, sizeof(empty_secret));
         };
     };
 
+    /* s2n_connection_export_secret */
+    {
+        /* Derives exporter secret on SERVER_FINISHED */
+        {
+            DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_SERVER),
+                    s2n_connection_ptr_free);
+            conn->secure->cipher_suite = &s2n_tls13_aes_128_gcm_sha256;
+            conn->actual_protocol_version = S2N_TLS13;
+            EXPECT_OK(s2n_connection_set_test_master_secret(conn, &test_secret));
+            EXPECT_BYTEARRAY_EQUAL(conn->secrets.version.tls13.exporter_master_secret,
+                    empty_secret, sizeof(empty_secret));
+
+            while (s2n_conn_get_current_message_type(conn) != SERVER_FINISHED) {
+                conn->handshake.message_number++;
+            }
+            EXPECT_OK(s2n_tls13_secrets_update(conn));
+
+            EXPECT_BYTEARRAY_NOT_EQUAL(conn->secrets.version.tls13.exporter_master_secret,
+                    empty_secret, sizeof(empty_secret));
+
+            /*
+             * s2n_connection_tls_exporter requires us to finish the handshake.
+             * The above is needed since s2n_tls13_secrets_update will only
+             * initialize when it sees the SERVER_FINISHED frame.
+             */
+            EXPECT_OK(s2n_skip_handshake(conn));
+
+            uint8_t output[32] = { 0 };
+            int result = s2n_connection_tls_exporter(
+                    conn,
+                    (const uint8_t *) "label",
+                    sizeof("label") - 1,
+                    (const uint8_t *) "context",
+                    sizeof("context") - 1,
+                    output,
+                    sizeof(output));
+            EXPECT_SUCCESS(result);
+            /*
+             * If updating this value, it's a good idea to make sure the update
+             * matches OpenSSL's SSL_export_keying_material. The easiest known
+             * way of doing that is building a simple client/server pair and
+             * calling the s2n and OpenSSL APIs after a handshake on both
+             * sides; you should get identical results with identical
+             * label/context parameters. This particular value though is not
+             * checked as its dependent on the s2n-specific test master secret.
+             */
+            S2N_BLOB_FROM_HEX(expected, "3a 72 eb 08 10 a3 69 f3 06 f2 77 11 70 ad d5 76 bd 21 15 \
+                    46 d4 c8 fb 80 1a 93 04 1e ac 59 aa 47");
+            EXPECT_EQUAL(sizeof(output), expected.size);
+            EXPECT_BYTEARRAY_EQUAL(output, expected.data, expected.size);
+        };
+    };
+
     END_TEST();
 }
