@@ -20,11 +20,13 @@
 #include <sys/param.h>
 #include <time.h>
 
+#include "api/unstable/fingerprint.h"
 #include "crypto/s2n_fips.h"
 #include "crypto/s2n_hash.h"
 #include "crypto/s2n_rsa_signing.h"
 #include "error/s2n_errno.h"
 #include "stuffer/s2n_stuffer.h"
+#include "tls/extensions/s2n_client_supported_groups.h"
 #include "tls/extensions/s2n_extension_list.h"
 #include "tls/extensions/s2n_server_key_share.h"
 #include "tls/s2n_alerts.h"
@@ -42,7 +44,7 @@
 
 struct s2n_client_hello *s2n_connection_get_client_hello(struct s2n_connection *conn)
 {
-    if (conn->client_hello.callback_invoked != 1) {
+    if (conn->client_hello.parsed != 1) {
         return NULL;
     }
 
@@ -243,16 +245,26 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
                           verify_len),
             S2N_ERR_BAD_MESSAGE);
 
-    /*
-     * We need to verify the client random separately
-     * because we erase it from the client hello during parsing.
-     * Compare the old value to the current value.
+    /* In the past, the s2n-tls client updated the client hello in ways not
+     * allowed by RFC8446: https://github.com/aws/s2n-tls/pull/3311
+     * Although the issue was addressed, its existence means that old versions
+     * of the s2n-tls client will fail this validation.
+     *
+     * So to avoid breaking old s2n-tls clients, we do not enforce this validation
+     * outside of tests. We continue to enforce it during tests to avoid regressions.
      */
-    RESULT_ENSURE(s2n_constant_time_equals(
-                          previous_client_random,
-                          conn->handshake_params.client_random,
-                          S2N_TLS_RANDOM_DATA_LEN),
-            S2N_ERR_BAD_MESSAGE);
+    if (s2n_in_test()) {
+        /*
+         * We need to verify the client random separately
+         * because we erase it from the client hello during parsing.
+         * Compare the old value to the current value.
+         */
+        RESULT_ENSURE(s2n_constant_time_equals(
+                              previous_client_random,
+                              conn->handshake_params.client_random,
+                              S2N_TLS_RANDOM_DATA_LEN),
+                S2N_ERR_BAD_MESSAGE);
+    }
 
     /*
      * Now enforce that the extensions also exactly match,
@@ -361,6 +373,12 @@ S2N_RESULT s2n_client_hello_parse_raw(struct s2n_client_hello *client_hello,
     /* legacy_version */
     RESULT_GUARD_POSIX(s2n_stuffer_read_bytes(in, client_protocol_version, S2N_TLS_PROTOCOL_VERSION_LEN));
 
+    /* Encode the version as a 1 byte representation of the two protocol version bytes, with the
+     * major version in the tens place and the minor version in the ones place. For example, the
+     * TLS 1.2 protocol version is 0x0303, which is encoded as S2N_TLS12 (33).
+     */
+    client_hello->legacy_version = (client_protocol_version[0] * 10) + client_protocol_version[1];
+
     /* random */
     RESULT_GUARD_POSIX(s2n_stuffer_erase_and_read_bytes(in, client_random, S2N_TLS_RANDOM_DATA_LEN));
 
@@ -381,10 +399,12 @@ S2N_RESULT s2n_client_hello_parse_raw(struct s2n_client_hello *client_hello,
     RESULT_ENSURE(cipher_suites != NULL, S2N_ERR_BAD_MESSAGE);
     RESULT_GUARD_POSIX(s2n_blob_init(&client_hello->cipher_suites, cipher_suites, cipher_suites_length));
 
-    /* legacy_compression_methods (ignored) */
-    uint8_t num_compression_methods = 0;
-    RESULT_GUARD_POSIX(s2n_stuffer_read_uint8(in, &num_compression_methods));
-    RESULT_GUARD_POSIX(s2n_stuffer_skip_read(in, num_compression_methods));
+    /* legacy_compression_methods */
+    uint8_t compression_methods_len = 0;
+    RESULT_GUARD_POSIX(s2n_stuffer_read_uint8(in, &compression_methods_len));
+    uint8_t *compression_methods = s2n_stuffer_raw_read(in, compression_methods_len);
+    RESULT_ENSURE(compression_methods != NULL, S2N_ERR_BAD_MESSAGE);
+    RESULT_GUARD_POSIX(s2n_blob_init(&client_hello->compression_methods, compression_methods, compression_methods_len));
 
     /* extensions */
     RESULT_GUARD_POSIX(s2n_extension_list_parse(in, &client_hello->extensions));
@@ -447,6 +467,11 @@ int s2n_parse_client_hello(struct s2n_connection *conn)
      *     https://tools.ietf.org/rfc/rfc8446#section-9.1
      *     A TLS-compliant application MUST support key exchange with secp256r1 (NIST P-256)
      *     and SHOULD support key exchange with X25519 [RFC7748]
+     *
+     *= https://tools.ietf.org/rfc/rfc4492#section-4
+     *# A client that proposes ECC cipher suites may choose not to include these extensions.
+     *# In this case, the server is free to choose any one of the elliptic curves or point formats listed in Section 5.
+     *
      */
     const struct s2n_ecc_preferences *ecc_pref = NULL;
     POSIX_GUARD(s2n_connection_get_ecc_preferences(conn, &ecc_pref));
@@ -579,9 +604,7 @@ int s2n_process_client_hello(struct s2n_connection *conn)
     }
 
     /* And set the signature and hash algorithm used for key exchange signatures */
-    POSIX_GUARD(s2n_choose_sig_scheme_from_peer_preference_list(conn,
-            &conn->handshake_params.client_sig_hash_algs,
-            &conn->handshake_params.conn_sig_scheme));
+    POSIX_GUARD_RESULT(s2n_signature_algorithm_select(conn));
 
     /* And finally, set the certs specified by the final auth + sig_alg combo. */
     POSIX_GUARD(s2n_select_certs_for_server_auth(conn, &conn->handshake_params.our_chain_and_key));
@@ -622,6 +645,7 @@ int s2n_client_hello_recv(struct s2n_connection *conn)
     /* Only parse the ClientHello once */
     if (!conn->client_hello.parsed) {
         POSIX_GUARD(s2n_parse_client_hello(conn));
+        /* Mark the collected client hello as available when parsing is done and before the client hello callback */
         conn->client_hello.parsed = true;
     }
 
@@ -631,7 +655,7 @@ int s2n_client_hello_recv(struct s2n_connection *conn)
      * callback state may have been cleared while parsing the second ClientHello.
      */
     if (!conn->client_hello.callback_invoked && !IS_HELLO_RETRY_HANDSHAKE(conn)) {
-        /* Mark the collected client hello as available when parsing is done and before the client hello callback */
+        /* Mark the client hello callback as invoked to avoid calling it again. */
         conn->client_hello.callback_invoked = true;
 
         /* Call client_hello_cb if exists, letting application to modify s2n_connection or swap s2n_config */
@@ -815,7 +839,7 @@ int s2n_sslv2_client_hello_recv(struct s2n_connection *conn)
     POSIX_GUARD(s2n_conn_find_name_matching_certs(conn));
 
     POSIX_GUARD(s2n_set_cipher_as_sslv2_server(conn, client_hello->cipher_suites.data, client_hello->cipher_suites.size / S2N_SSLv2_CIPHER_SUITE_LEN));
-    POSIX_GUARD(s2n_choose_default_sig_scheme(conn, &conn->handshake_params.conn_sig_scheme, S2N_SERVER));
+    POSIX_GUARD_RESULT(s2n_signature_algorithm_select(conn));
     POSIX_GUARD(s2n_select_certs_for_server_auth(conn, &conn->handshake_params.our_chain_and_key));
 
     S2N_ERROR_IF(session_id_length > s2n_stuffer_data_available(in), S2N_ERR_BAD_MESSAGE);
@@ -846,7 +870,7 @@ int s2n_client_hello_get_parsed_extension(s2n_tls_extension_type extension_type,
     POSIX_GUARD(s2n_extension_supported_iana_value_to_id(extension_type, &extension_type_id));
 
     s2n_parsed_extension *found_parsed_extension = &parsed_extension_list->parsed_extensions[extension_type_id];
-    POSIX_ENSURE_REF(found_parsed_extension->extension.data);
+    POSIX_ENSURE(found_parsed_extension->extension.data, S2N_ERR_EXTENSION_NOT_RECEIVED);
     POSIX_ENSURE(found_parsed_extension->extension_type == extension_type, S2N_ERR_INVALID_PARSED_EXTENSIONS);
 
     *parsed_extension = found_parsed_extension;
@@ -901,6 +925,43 @@ int s2n_client_hello_get_session_id(struct s2n_client_hello *ch, uint8_t *out, u
     return S2N_SUCCESS;
 }
 
+int s2n_client_hello_get_compression_methods_length(struct s2n_client_hello *ch, uint32_t *out_length)
+{
+    POSIX_ENSURE_REF(ch);
+    POSIX_ENSURE_REF(out_length);
+    *out_length = ch->compression_methods.size;
+    return S2N_SUCCESS;
+}
+
+int s2n_client_hello_get_compression_methods(struct s2n_client_hello *ch, uint8_t *list, uint32_t list_length, uint32_t *out_length)
+{
+    POSIX_ENSURE_REF(ch);
+    POSIX_ENSURE_REF(list);
+    POSIX_ENSURE_REF(out_length);
+
+    POSIX_ENSURE(list_length >= ch->compression_methods.size, S2N_ERR_INSUFFICIENT_MEM_SIZE);
+    POSIX_CHECKED_MEMCPY(list, ch->compression_methods.data, ch->compression_methods.size);
+    *out_length = ch->compression_methods.size;
+    return S2N_SUCCESS;
+}
+
+int s2n_client_hello_get_legacy_protocol_version(struct s2n_client_hello *ch, uint8_t *out)
+{
+    POSIX_ENSURE_REF(ch);
+    POSIX_ENSURE_REF(out);
+    *out = ch->legacy_version;
+    return S2N_SUCCESS;
+}
+
+int s2n_client_hello_get_legacy_record_version(struct s2n_client_hello *ch, uint8_t *out)
+{
+    POSIX_ENSURE_REF(ch);
+    POSIX_ENSURE_REF(out);
+    POSIX_ENSURE(ch->record_version_recorded, S2N_ERR_INVALID_ARGUMENT);
+    *out = ch->legacy_record_version;
+    return S2N_SUCCESS;
+}
+
 static S2N_RESULT s2n_client_hello_get_raw_extension(uint16_t extension_iana,
         struct s2n_blob *raw_extensions, struct s2n_blob *extension)
 {
@@ -954,3 +1015,34 @@ int s2n_client_hello_has_extension(struct s2n_client_hello *ch, uint16_t extensi
     }
     return S2N_SUCCESS;
 }
+
+int s2n_client_hello_get_supported_groups(struct s2n_client_hello *ch, uint16_t *groups,
+        uint16_t groups_count_max, uint16_t *groups_count_out)
+{
+    POSIX_ENSURE_REF(groups_count_out);
+    *groups_count_out = 0;
+    POSIX_ENSURE_REF(ch);
+    POSIX_ENSURE_REF(groups);
+
+    s2n_parsed_extension *supported_groups_extension = NULL;
+    POSIX_GUARD(s2n_client_hello_get_parsed_extension(S2N_EXTENSION_SUPPORTED_GROUPS, &ch->extensions, &supported_groups_extension));
+    POSIX_ENSURE_REF(supported_groups_extension);
+
+    struct s2n_stuffer extension_stuffer = { 0 };
+    POSIX_GUARD(s2n_stuffer_init_written(&extension_stuffer, &supported_groups_extension->extension));
+
+    uint16_t supported_groups_count = 0;
+    POSIX_GUARD_RESULT(s2n_supported_groups_parse_count(&extension_stuffer, &supported_groups_count));
+    POSIX_ENSURE(supported_groups_count <= groups_count_max, S2N_ERR_INSUFFICIENT_MEM_SIZE);
+
+    for (size_t i = 0; i < supported_groups_count; i++) {
+        /* s2n_stuffer_read_uint16 is used to read each of the supported groups in network-order
+         * endianness.
+         */
+        POSIX_GUARD(s2n_stuffer_read_uint16(&extension_stuffer, &groups[i]));
+    }
+
+    *groups_count_out = supported_groups_count;
+
+    return S2N_SUCCESS;
+}
