@@ -104,12 +104,12 @@ struct s2n_handshake_parameters {
     /* Signature/hash algorithm pairs offered by the client in the signature_algorithms extension */
     struct s2n_sig_scheme_list client_sig_hash_algs;
     /* Signature scheme chosen by the server */
-    struct s2n_signature_scheme conn_sig_scheme;
+    const struct s2n_signature_scheme *server_cert_sig_scheme;
 
     /* Signature/hash algorithm pairs offered by the server in the certificate request */
     struct s2n_sig_scheme_list server_sig_hash_algs;
     /* Signature scheme chosen by the client */
-    struct s2n_signature_scheme client_cert_sig_scheme;
+    const struct s2n_signature_scheme *client_cert_sig_scheme;
 
     /* The cert chain we will send the peer. */
     struct s2n_cert_chain_and_key *our_chain_and_key;
