@@ -17,97 +17,22 @@
 #include "tls/s2n_alerts.h"
 #include "tls/s2n_connection.h"
 #include "tls/s2n_tls.h"
+#include "utils/s2n_atomic.h"
 #include "utils/s2n_safety.h"
 
-typedef enum {
-    S2N_SHUTDOWN_RDWR,
-    S2N_SHUTDOWN_WR,
-} s2n_shutdown_how;
-
-static bool s2n_error_alert_received(struct s2n_connection *conn)
+static bool s2n_shutdown_expect_close_notify(struct s2n_connection *conn)
 {
-    /* We don't check s2n_connection_get_alert() or s2n_stuffer_data_available()
-     * because of https://github.com/aws/s2n-tls/issues/3933.
-     * We need to check if the stuffer contains an alert, regardless of its
-     * read state.
-     */
-    if (conn->alert_in.write_cursor == 0) {
+    /* No close_notify expected if we already received an error instead */
+    if (s2n_atomic_flag_test(&conn->error_alert_received)) {
         return false;
     }
-    /* Verify that the stuffer doesn't just contain a close_notify alert */
-    if (conn->close_notify_received) {
+
+    /* No close_notify expected if we sent an error instead of a close_notify */
+    if (conn->writer_alert_out || conn->reader_alert_out) {
         return false;
     }
-    return true;
-}
 
-static bool s2n_error_alert_sent(struct s2n_connection *conn)
-{
-    /* Sending an alert always sets conn->write_closing: see s2n_flush() */
-    if (!conn->write_closing) {
-        return false;
-    }
-    /* Verify that the alert sent wasn't just a close_notify */
-    if (conn->close_notify_queued) {
-        return false;
-    }
-    return true;
-}
-
-int s2n_shutdown_impl(struct s2n_connection *conn, s2n_shutdown_how how,
-        s2n_blocked_status *blocked)
-{
-    POSIX_ENSURE_REF(conn);
-    POSIX_ENSURE_REF(blocked);
-
-    /* Treat this call as a no-op if already wiped */
-    if (conn->send == NULL && conn->recv == NULL) {
-        return S2N_SUCCESS;
-    }
-
-    /* Treat this call as a no-op if an error alert was already received.
-     * Error alerts close the connection without any exchange of close_notify alerts. */
-    if (s2n_error_alert_received(conn)) {
-        return S2N_SUCCESS;
-    }
-
-    /* Enforce blinding.
-     * If an application is using self-service blinding, ensure that they have
-     * waited the required time before triggering the close_notify alert.
-     */
-    uint64_t elapsed = 0;
-    POSIX_GUARD_RESULT(s2n_timer_elapsed(conn->config, &conn->write_timer, &elapsed));
-    S2N_ERROR_IF(elapsed < conn->delay, S2N_ERR_SHUTDOWN_PAUSED);
-
-    /* Flush any outstanding data or alerts */
-    POSIX_GUARD(s2n_flush(conn, blocked));
-
-    /* Error alerts close the connection without any exchange of close_notify alerts.
-     * We need to check after flushing to account for any pending alerts.
-     */
-    if (s2n_error_alert_sent(conn)) {
-        conn->read_closed = 1;
-        return S2N_SUCCESS;
-    }
-
-    /**
-     *= https://tools.ietf.org/rfc/rfc8446#section-6.1
-     *# Each party MUST send a "close_notify" alert before closing its write
-     *# side of the connection, unless it has already sent some error alert.
-     */
-    if (!conn->close_notify_queued) {
-        POSIX_GUARD(s2n_queue_writer_close_alert_warning(conn));
-        conn->close_notify_queued = 1;
-        POSIX_GUARD(s2n_flush(conn, blocked));
-    }
-
-    /* If we're only closing the write side, then we've succeeded. */
-    if (how == S2N_SHUTDOWN_WR) {
-        return S2N_SUCCESS;
-    }
-
-    /*
-     * The purpose of the peer responding to our close_notify
+    /* The purpose of the peer responding to our close_notify
      * with its own close_notify is to prevent application data truncation.
      * However, application data is not a concern during the handshake.
      *
@@ -116,6 +41,76 @@ int s2n_shutdown_impl(struct s2n_connection *conn, s2n_shutdown_how how,
      * and unnecessary blinding.
      */
     if (!s2n_handshake_is_complete(conn)) {
+        return false;
+    }
+
+    /* QUIC does not use TLS alerts */
+    if (conn->quic_enabled) {
+        return false;
+    }
+
+    return true;
+}
+
+int s2n_shutdown_send(struct s2n_connection *conn, s2n_blocked_status *blocked)
+{
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(blocked);
+    *blocked = S2N_NOT_BLOCKED;
+
+    /* Treat this call as a no-op if already wiped.
+     * This should probably be an error, but wasn't in the past so is left as-is
+     * for backwards compatibility.
+     */
+    if (conn->send == NULL && conn->recv == NULL) {
+        return S2N_SUCCESS;
+    }
+
+    /* Flush any outstanding data */
+    s2n_atomic_flag_set(&conn->write_closed);
+    POSIX_GUARD(s2n_flush(conn, blocked));
+
+    /* For a connection closed due to receiving an alert, we don't send anything. */
+    if (s2n_atomic_flag_test(&conn->error_alert_received)) {
+        return S2N_SUCCESS;
+    }
+
+    /* If we've already sent an alert, don't send another. */
+    if (conn->alert_sent) {
+        return S2N_SUCCESS;
+    }
+
+    /* Enforce blinding.
+     * If an application is using self-service blinding, ensure that they have
+     * waited the required time before triggering any alerts.
+     */
+    uint64_t elapsed = 0;
+    POSIX_GUARD_RESULT(s2n_timer_elapsed(conn->config, &conn->write_timer, &elapsed));
+    S2N_ERROR_IF(elapsed < conn->delay, S2N_ERR_SHUTDOWN_PAUSED);
+
+    /**
+     *= https://tools.ietf.org/rfc/rfc8446#section-6.1
+     *# Each party MUST send a "close_notify" alert before closing its write
+     *# side of the connection, unless it has already sent some error alert.
+     */
+    POSIX_GUARD_RESULT(s2n_alerts_write_error_or_close_notify(conn));
+    POSIX_GUARD(s2n_flush(conn, blocked));
+    return S2N_SUCCESS;
+}
+
+int s2n_shutdown(struct s2n_connection *conn, s2n_blocked_status *blocked)
+{
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(blocked);
+    *blocked = S2N_NOT_BLOCKED;
+
+    /* If necessary, send an alert to indicate shutdown. */
+    POSIX_GUARD(s2n_shutdown_send(conn, blocked));
+
+    /* If we don't expect a close_notify from our peer,
+     * just ensure that the connection is marked closed.
+     */
+    if (!s2n_shutdown_expect_close_notify(conn)) {
         POSIX_GUARD_RESULT(s2n_connection_set_closed(conn));
         *blocked = S2N_NOT_BLOCKED;
         return S2N_SUCCESS;
@@ -125,31 +120,19 @@ int s2n_shutdown_impl(struct s2n_connection *conn, s2n_shutdown_how how,
     uint8_t record_type = 0;
     int isSSLv2 = false;
     *blocked = S2N_BLOCKED_ON_READ;
-    while (!conn->close_notify_received) {
+    while (!s2n_atomic_flag_test(&conn->close_notify_received)) {
+        /* Reset IO. Make sure we do this before attempting to read a record in
+         * case a previous failed read left IO in a bad state.
+         */
+        POSIX_GUARD_RESULT(s2n_record_wipe(conn));
+
         POSIX_GUARD(s2n_read_full_record(conn, &record_type, &isSSLv2));
         POSIX_ENSURE(!isSSLv2, S2N_ERR_BAD_MESSAGE);
         if (record_type == TLS_ALERT) {
             POSIX_GUARD(s2n_process_alert_fragment(conn));
         }
-
-        /* Wipe and keep trying */
-        POSIX_GUARD(s2n_stuffer_wipe(&conn->header_in));
-        POSIX_GUARD(s2n_stuffer_wipe(&conn->in));
-        conn->in_status = ENCRYPTED;
     }
 
     *blocked = S2N_NOT_BLOCKED;
     return S2N_SUCCESS;
 }
-
-int s2n_shutdown_send(struct s2n_connection *conn, s2n_blocked_status *blocked)
-{
-    POSIX_GUARD(s2n_shutdown_impl(conn, S2N_SHUTDOWN_WR, blocked));
-    return S2N_SUCCESS;
-}
-
-int s2n_shutdown(struct s2n_connection *conn, s2n_blocked_status *blocked)
-{
-    POSIX_GUARD(s2n_shutdown_impl(conn, S2N_SHUTDOWN_RDWR, blocked));
-    return S2N_SUCCESS;
-}
