@@ -15,14 +15,13 @@
 
 #include "utils/s2n_safety.h"
 
-void *s2n_ensure_memcpy_trace(void *restrict to, const void *restrict from, size_t size, const char *debug_str)
+void *s2n_ensure_memcpy_trace(void *restrict to, const void *restrict from, size_t size)
 {
-    if (to == NULL || from == NULL) {
-        s2n_errno = S2N_ERR_NULL;
-        s2n_debug_str = debug_str;
-        return NULL;
-    }
+    PTR_ENSURE_REF(to);
+    PTR_ENSURE_REF(from);
 
     /* use memmove instead of memcpy since it'll handle overlapping regions and not result in UB */
-    return memmove(to, from, size);
+    void *result = memmove(to, from, size);
+    PTR_ENSURE_REF(result);
+    return result;
 }
