@@ -10,6 +10,8 @@
 extern const char *s_capsule_name_hash;
 /** Name string for hmac capsule. */
 extern const char *s_capsule_name_hmac;
+/** Name string for rsa capsule. */
+extern const char *s_capsule_name_rsa;
 
 PyObject *aws_py_sha1_new(PyObject *self, PyObject *args);
 PyObject *aws_py_sha256_new(PyObject *self, PyObject *args);
@@ -27,4 +29,12 @@ PyObject *aws_py_sha256_compute(PyObject *self, PyObject *args);
 PyObject *aws_py_md5_compute(PyObject *self, PyObject *args);
 PyObject *aws_py_sha256_hmac_compute(PyObject *self, PyObject *args);
 
+PyObject *aws_py_rsa_private_key_from_pem_data(PyObject *self, PyObject *args);
+PyObject *aws_py_rsa_public_key_from_pem_data(PyObject *self, PyObject *args);
+
+PyObject *aws_py_rsa_encrypt(PyObject *self, PyObject *args);
+PyObject *aws_py_rsa_decrypt(PyObject *self, PyObject *args);
+PyObject *aws_py_rsa_sign(PyObject *self, PyObject *args);
+PyObject *aws_py_rsa_verify(PyObject *self, PyObject *args);
+
 #endif /* AWS_CRT_PYTHON_CRYPTO_H */
