@@ -3,6 +3,7 @@
  * SPDX-License-Identifier: Apache-2.0.
  */
 #include "mqtt_client_connection.h"
+#include "mqtt5_client.h"
 
 #include "http.h"
 #include "io.h"
@@ -56,7 +57,6 @@ struct mqtt_connection_binding {
 };
 
 static void s_mqtt_python_connection_finish_destruction(struct mqtt_connection_binding *py_connection) {
-    aws_mqtt_client_connection_release(py_connection->native);
 
     Py_DECREF(py_connection->self_proxy);
     Py_DECREF(py_connection->client);
@@ -65,11 +65,20 @@ static void s_mqtt_python_connection_finish_destruction(struct mqtt_connection_b
     aws_mem_release(aws_py_get_allocator(), py_connection);
 }
 
-static void s_mqtt_python_connection_destructor_on_disconnect(
-    struct aws_mqtt_client_connection *connection,
-    void *userdata) {
+static void s_start_destroy_native(struct mqtt_connection_binding *py_connection) {
+    if (py_connection == NULL || py_connection->native == NULL) {
+        return;
+    }
+
+    aws_mqtt_client_connection_release(py_connection->native);
+}
+
+static void s_mqtt_python_connection_termination(void *userdata) {
+
+    if (userdata == NULL) {
+        return; // The binding is dead - skip!
+    }
 
-    (void)connection;
     struct mqtt_connection_binding *py_connection = userdata;
 
     PyGILState_STATE state;
@@ -81,23 +90,101 @@ static void s_mqtt_python_connection_destructor_on_disconnect(
     PyGILState_Release(state);
 }
 
+static void s_mqtt_python_connection_destructor_on_disconnect(
+    struct aws_mqtt_client_connection *connection,
+    void *user_data) {
+    if (connection == NULL || user_data == NULL) {
+        return; // The connection is dead - skip!
+    }
+
+    struct mqtt_connection_binding *py_connection = user_data;
+    PyGILState_STATE state;
+    if (aws_py_gilstate_ensure(&state)) {
+        return; /* Python has shut down. Nothing matters anymore, but don't crash */
+    }
+    s_start_destroy_native(py_connection);
+    PyGILState_Release(state);
+}
+
 static void s_mqtt_python_connection_destructor(PyObject *connection_capsule) {
 
     struct mqtt_connection_binding *py_connection =
         PyCapsule_GetPointer(connection_capsule, s_capsule_name_mqtt_client_connection);
-    assert(py_connection);
+    AWS_FATAL_ASSERT(py_connection);
+    AWS_FATAL_ASSERT(py_connection->native);
+
+    /* This is the destructor from Python - so we can ignore the closed callback here */
+    aws_mqtt_client_connection_set_connection_closed_handler(py_connection->native, NULL, NULL);
 
     if (aws_mqtt_client_connection_disconnect(
             py_connection->native, s_mqtt_python_connection_destructor_on_disconnect, py_connection)) {
-
-        /* If this returns an error, we should immediately destroy the connection */
-        s_mqtt_python_connection_finish_destruction(py_connection);
+        /* If we already disconnected, we should immediately release the native connection */
+        s_start_destroy_native(py_connection);
     }
 }
 
+static void s_on_connection_success(
+    struct aws_mqtt_client_connection *connection,
+    enum aws_mqtt_connect_return_code return_code,
+    bool session_present,
+    void *user_data) {
+
+    if (connection == NULL || user_data == NULL) {
+        return; // The connection is dead - skip!
+    }
+
+    struct mqtt_connection_binding *py_connection = user_data;
+
+    PyGILState_STATE state;
+    if (aws_py_gilstate_ensure(&state)) {
+        return; /* Python has shut down. Nothing matters anymore, but don't crash */
+    }
+
+    PyObject *self = PyWeakref_GetObject(py_connection->self_proxy); /* borrowed reference */
+    if (self != Py_None) {
+        PyObject *success_result =
+            PyObject_CallMethod(self, "_on_connection_success", "(iN)", return_code, PyBool_FromLong(session_present));
+        if (success_result) {
+            Py_DECREF(success_result);
+        } else {
+            PyErr_WriteUnraisable(PyErr_Occurred());
+        }
+    }
+
+    PyGILState_Release(state);
+}
+
+static void s_on_connection_failure(struct aws_mqtt_client_connection *connection, int error_code, void *user_data) {
+
+    if (connection == NULL || user_data == NULL) {
+        return; // The connection is dead - skip!
+    }
+
+    struct mqtt_connection_binding *py_connection = user_data;
+
+    PyGILState_STATE state;
+    if (aws_py_gilstate_ensure(&state)) {
+        return; /* Python has shut down. Nothing matters anymore, but don't crash */
+    }
+
+    PyObject *self = PyWeakref_GetObject(py_connection->self_proxy); /* borrowed reference */
+    if (self != Py_None) {
+        PyObject *success_result = PyObject_CallMethod(self, "_on_connection_failure", "(i)", error_code);
+        if (success_result) {
+            Py_DECREF(success_result);
+        } else {
+            PyErr_WriteUnraisable(PyErr_Occurred());
+        }
+    }
+
+    PyGILState_Release(state);
+}
+
 static void s_on_connection_interrupted(struct aws_mqtt_client_connection *connection, int error_code, void *userdata) {
 
-    (void)connection;
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     struct mqtt_connection_binding *py_connection = userdata;
 
@@ -126,6 +213,10 @@ static void s_on_connection_resumed(
     bool session_present,
     void *userdata) {
 
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
+
     (void)connection;
 
     struct mqtt_connection_binding *py_connection = userdata;
@@ -150,6 +241,36 @@ static void s_on_connection_resumed(
     PyGILState_Release(state);
 }
 
+static void s_on_connection_closed(
+    struct aws_mqtt_client_connection *connection,
+    struct on_connection_closed_data *data,
+    void *userdata) {
+
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
+    (void)data; // Not used for anything currently, but in the future it could be.
+
+    PyGILState_STATE state;
+    if (aws_py_gilstate_ensure(&state)) {
+        return; /* Python has shut down. Nothing matters anymore, but don't crash */
+    }
+
+    struct mqtt_connection_binding *py_connection = userdata;
+    /* Ensure that python class is still alive */
+    PyObject *self = PyWeakref_GetObject(py_connection->self_proxy); /* borrowed reference */
+    if (self != Py_None) {
+        PyObject *result = PyObject_CallMethod(self, "_on_connection_closed", "()");
+        if (result) {
+            Py_DECREF(result);
+        } else {
+            PyErr_WriteUnraisable(PyErr_Occurred());
+        }
+    }
+
+    PyGILState_Release(state);
+}
+
 PyObject *aws_py_mqtt_client_connection_new(PyObject *self, PyObject *args) {
     (void)self;
 
@@ -158,11 +279,21 @@ PyObject *aws_py_mqtt_client_connection_new(PyObject *self, PyObject *args) {
     PyObject *self_py;
     PyObject *client_py;
     PyObject *use_websocket_py;
-    if (!PyArg_ParseTuple(args, "OOO", &self_py, &client_py, &use_websocket_py)) {
+    unsigned char client_version;
+    if (!PyArg_ParseTuple(args, "OOOb", &self_py, &client_py, &use_websocket_py, &client_version)) {
+        return NULL;
+    }
+
+    void *client = NULL;
+    if (client_version == 3) {
+        client = aws_py_get_mqtt_client(client_py);
+    } else if (client_version == 5) {
+        client = aws_py_get_mqtt5_client(client_py);
+    } else {
+        PyErr_SetString(PyExc_TypeError, "Mqtt Client version not supported. Failed to create connection.");
         return NULL;
     }
 
-    struct aws_mqtt_client *client = aws_py_get_mqtt_client(client_py);
     if (!client) {
         return NULL;
     }
@@ -175,11 +306,28 @@ PyObject *aws_py_mqtt_client_connection_new(PyObject *self, PyObject *args) {
     }
 
     /* From hereon, we need to clean up if errors occur */
+    PyObject *self_proxy = NULL;
 
-    py_connection->native = aws_mqtt_client_connection_new(client);
+    if (client_version == 3) {
+        py_connection->native = aws_mqtt_client_connection_new(client);
+    } else if (client_version == 5) {
+        py_connection->native = aws_mqtt_client_connection_new_from_mqtt5_client(client);
+    }
     if (!py_connection->native) {
         PyErr_SetAwsLastError();
-        goto connection_new_failed;
+        goto on_error;
+    }
+
+    if (aws_mqtt_client_connection_set_connection_termination_handler(
+            py_connection->native, s_mqtt_python_connection_termination, py_connection)) {
+        PyErr_SetAwsLastError();
+        goto on_error;
+    }
+
+    if (aws_mqtt_client_connection_set_connection_result_handlers(
+            py_connection->native, s_on_connection_success, py_connection, s_on_connection_failure, py_connection)) {
+        PyErr_SetAwsLastError();
+        goto on_error;
     }
 
     if (aws_mqtt_client_connection_set_connection_interruption_handlers(
@@ -190,7 +338,13 @@ PyObject *aws_py_mqtt_client_connection_new(PyObject *self, PyObject *args) {
             py_connection)) {
 
         PyErr_SetAwsLastError();
-        goto set_interruption_failed;
+        goto on_error;
+    }
+
+    if (aws_mqtt_client_connection_set_connection_closed_handler(
+            py_connection->native, s_on_connection_closed, py_connection)) {
+        PyErr_SetAwsLastError();
+        goto on_error;
     }
 
     if (PyObject_IsTrue(use_websocket_py)) {
@@ -202,23 +356,22 @@ PyObject *aws_py_mqtt_client_connection_new(PyObject *self, PyObject *args) {
                 NULL /*validator userdata*/)) {
 
             PyErr_SetAwsLastError();
-            goto use_websockets_failed;
+            goto on_error;
         }
     }
 
-    PyObject *self_proxy = PyWeakref_NewProxy(self_py, NULL);
+    self_proxy = PyWeakref_NewProxy(self_py, NULL);
     if (!self_proxy) {
-        goto proxy_new_failed;
+        goto on_error;
     }
 
     PyObject *capsule =
         PyCapsule_New(py_connection, s_capsule_name_mqtt_client_connection, s_mqtt_python_connection_destructor);
     if (!capsule) {
-        goto capsule_new_failed;
+        goto on_error;
     }
 
     /* From hereon, nothing will fail */
-
     py_connection->self_proxy = self_proxy;
 
     py_connection->client = client_py;
@@ -226,13 +379,9 @@ PyObject *aws_py_mqtt_client_connection_new(PyObject *self, PyObject *args) {
 
     return capsule;
 
-capsule_new_failed:
-    Py_DECREF(self_proxy);
-proxy_new_failed:
-use_websockets_failed:
-set_interruption_failed:
+on_error:
+    Py_XDECREF(self_proxy);
     aws_mqtt_client_connection_release(py_connection->native);
-connection_new_failed:
     aws_mem_release(allocator, py_connection);
     return NULL;
 }
@@ -253,15 +402,18 @@ static void s_on_connect(
     bool session_present,
     void *user_data) {
 
-    (void)connection;
+    if (connection == NULL || user_data == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     struct mqtt_connection_binding *py_connection = user_data;
 
+    PyGILState_STATE state;
+    if (aws_py_gilstate_ensure(&state)) {
+        return; /* Python has shut down. Nothing matters anymore, but don't crash */
+    }
+
     if (py_connection->on_connect) {
-        PyGILState_STATE state;
-        if (aws_py_gilstate_ensure(&state)) {
-            return; /* Python has shut down. Nothing matters anymore, but don't crash */
-        }
 
         PyObject *callback = py_connection->on_connect;
         py_connection->on_connect = NULL;
@@ -275,15 +427,19 @@ static void s_on_connect(
         }
 
         Py_XDECREF(callback);
-
-        PyGILState_Release(state);
     }
+
+    PyGILState_Release(state);
 }
 
 /* If unsuccessful, false is returned and a Python error has been set */
 bool s_set_will(struct aws_mqtt_client_connection *connection, PyObject *will) {
     assert(will && (will != Py_None));
 
+    if (connection == NULL) {
+        return false; // The connection is dead - skip!
+    }
+
     bool success = false;
 
     /* These references all need to be cleaned up before function returns */
@@ -489,7 +645,7 @@ PyObject *aws_py_mqtt_client_connection_connect(PyObject *self, PyObject *args)
     Py_ssize_t client_id_len;
     const char *server_name;
     Py_ssize_t server_name_len;
-    uint16_t port_number;
+    uint32_t port_number;
     PyObject *socket_options_py;
     PyObject *tls_ctx_py;
     uint64_t reconnect_min_timeout_secs;
@@ -507,7 +663,7 @@ PyObject *aws_py_mqtt_client_connection_connect(PyObject *self, PyObject *args)
     PyObject *proxy_options_py;
     if (!PyArg_ParseTuple(
             args,
-            "Os#s#HOOKKHIIOz#z#OOO",
+            "Os#s#IOOKKHIIOz#z#OOO",
             &impl_capsule,
             &client_id,
             &client_id_len,
@@ -697,7 +853,10 @@ static void s_publish_complete(
     uint16_t packet_id,
     int error_code,
     void *userdata) {
-    (void)connection;
+
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     struct publish_complete_userdata *metadata = userdata;
     assert(metadata);
@@ -805,7 +964,9 @@ static void s_subscribe_callback(
     bool retain,
     void *user_data) {
 
-    (void)connection;
+    if (connection == NULL || user_data == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     PyObject *callback = user_data;
     if (callback == Py_None) {
@@ -858,7 +1019,9 @@ static void s_suback_callback(
     int error_code,
     void *userdata) {
 
-    (void)connection;
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     PyObject *callback = userdata;
     AWS_FATAL_ASSERT(callback && callback != Py_None);
@@ -967,7 +1130,10 @@ static void s_unsuback_callback(
     uint16_t packet_id,
     int error_code,
     void *userdata) {
-    (void)connection;
+
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     PyObject *callback = userdata;
 
@@ -1029,7 +1195,9 @@ static void s_suback_multi_callback(
     int error_code,
     void *userdata) {
 
-    (void)connection;
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     /* These must be DECREF'd when function ends */
     PyObject *callback = userdata;
@@ -1063,7 +1231,7 @@ static void s_suback_multi_callback(
             goto done_prepping_args;
         }
 
-        PyList_SET_ITEM(topic_qos_list, i, tuple); /* Steals reference to tuple */
+        PyList_SetItem(topic_qos_list, i, tuple); /* Steals reference to tuple */
     }
 
 done_prepping_args:;
@@ -1124,7 +1292,9 @@ PyObject *aws_py_mqtt_client_connection_resubscribe_existing_topics(PyObject *se
 
 static void s_on_disconnect(struct aws_mqtt_client_connection *connection, void *user_data) {
 
-    (void)connection;
+    if (connection == NULL || user_data == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     PyObject *on_disconnect = user_data;
 
@@ -1173,6 +1343,10 @@ PyObject *aws_py_mqtt_client_connection_disconnect(PyObject *self, PyObject *arg
     Py_RETURN_NONE;
 }
 
+/*******************************************************************************
+ * Client Statistics
+ ******************************************************************************/
+
 PyObject *aws_py_mqtt_client_connection_get_stats(PyObject *self, PyObject *args) {
     (void)self;
     bool success = false;
@@ -1202,23 +1376,35 @@ PyObject *aws_py_mqtt_client_connection_get_stats(PyObject *self, PyObject *args
         goto done;
     }
 
-    PyTuple_SET_ITEM(result, 0, PyLong_FromUnsignedLongLong((unsigned long long)stats.incomplete_operation_count));
-    if (PyTuple_GET_ITEM(result, 0) == NULL) {
+    PyTuple_SetItem(
+        result,
+        0,
+        PyLong_FromUnsignedLongLong((unsigned long long)stats.incomplete_operation_count)); /* Steals a reference */
+    if (PyTuple_GetItem(result, 0) == NULL) {                                               /* Borrowed reference */
         goto done;
     }
 
-    PyTuple_SET_ITEM(result, 1, PyLong_FromUnsignedLongLong((unsigned long long)stats.incomplete_operation_size));
-    if (PyTuple_GET_ITEM(result, 1) == NULL) {
+    PyTuple_SetItem(
+        result,
+        1,
+        PyLong_FromUnsignedLongLong((unsigned long long)stats.incomplete_operation_size)); /* Steals a reference */
+    if (PyTuple_GetItem(result, 1) == NULL) {                                              /* Borrowed reference */
         goto done;
     }
 
-    PyTuple_SET_ITEM(result, 2, PyLong_FromUnsignedLongLong((unsigned long long)stats.unacked_operation_count));
-    if (PyTuple_GET_ITEM(result, 2) == NULL) {
+    PyTuple_SetItem(
+        result,
+        2,
+        PyLong_FromUnsignedLongLong((unsigned long long)stats.unacked_operation_count)); /* Steals a reference */
+    if (PyTuple_GetItem(result, 2) == NULL) {                                            /* Borrowed reference */
         goto done;
     }
 
-    PyTuple_SET_ITEM(result, 3, PyLong_FromUnsignedLongLong((unsigned long long)stats.unacked_operation_size));
-    if (PyTuple_GET_ITEM(result, 3) == NULL) {
+    PyTuple_SetItem(
+        result,
+        3,
+        PyLong_FromUnsignedLongLong((unsigned long long)stats.unacked_operation_size)); /* Steals a reference */
+    if (PyTuple_GetItem(result, 3) == NULL) {                                           /* Borrowed reference */
         goto done;
     }
 
